@@ -1,5 +1,5 @@
 /* io.c: Common serialization routines for libgzochi-common
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -17,7 +17,21 @@
 
 #include "io.h"
 
-int gzochi_common_io_read_int (unsigned char *bytes, int offset)
+long long
+gzochi_common_io_read_long (const unsigned char *bytes, int offset)
+{
+  return ((long long) bytes[offset] << 56)
+    + ((long long) bytes[offset + 1] << 48)
+    + ((long long) bytes[offset + 2] << 40)
+    + ((long long) bytes[offset + 3] << 32)
+    + ((long long) bytes[offset + 4] << 24)
+    + ((long long) bytes[offset + 5] << 16)
+    + ((long long) bytes[offset + 6] << 8)
+    + ((long long) bytes[offset + 7]);
+}
+
+int
+gzochi_common_io_read_int (const unsigned char *bytes, int offset)
 {
   return (bytes[offset] << 24)
     + (bytes[offset + 1] << 16)
@@ -25,12 +39,27 @@ int gzochi_common_io_read_int (unsigned char *bytes, int offset)
     + (bytes[offset + 3]);
 }
 
-short gzochi_common_io_read_short (unsigned char *bytes, int offset)
+short
+gzochi_common_io_read_short (const unsigned char *bytes, int offset)
 {
   return (bytes[offset] << 8) + bytes[offset + 1];
 }
 
-void gzochi_common_io_write_int (int val, unsigned char *bytes, int offset)
+void
+gzochi_common_io_write_long (long long val, unsigned char *bytes, int offset)
+{
+  bytes[offset] = val >> 56;
+  bytes[offset + 1] = val >> 48 & 0xff;
+  bytes[offset + 2] = val >> 40 & 0xff;
+  bytes[offset + 3] = val >> 32 & 0xff;
+  bytes[offset + 4] = val >> 24 & 0xff;
+  bytes[offset + 5] = val >> 16 & 0xff;
+  bytes[offset + 6] = val >> 8 & 0xff;
+  bytes[offset + 7] = val & 0xff;
+}
+
+void
+gzochi_common_io_write_int (int val, unsigned char *bytes, int offset)
 {
   bytes[offset] = val >> 24;
   bytes[offset + 1] = val >> 16 & 0xff;
@@ -38,7 +67,8 @@ void gzochi_common_io_write_int (int val, unsigned char *bytes, int offset)
   bytes[offset + 3] = val & 0xff;
 }
 
-void gzochi_common_io_write_short (short val, unsigned char *bytes, int offset)
+void
+gzochi_common_io_write_short (short val, unsigned char *bytes, int offset)
 {
   bytes[offset] = val >> 8 & 0xff;
   bytes[offset + 1] = val & 0xff;
