@@ -1,5 +1,5 @@
 /* channel.h: Prototypes and declarations for channel.c
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,33 +19,47 @@
 #define GZOCHID_CHANNEL_H
 
 #include <glib.h>
-#include <gmp.h>
 
 #include "app.h"
+#include "durable-task.h"
 #include "io.h"
 #include "session.h"
 #include "task.h"
 
-typedef struct _gzochid_channel
-{
-  char *name;
-
-  unsigned char *id;
-  short id_len;
-
-  GSequence *sessions;
-
-  mpz_t oid;
-  mpz_t scm_oid;
-} gzochid_channel;
+typedef struct _gzochid_channel gzochid_channel;
 
 extern gzochid_io_serialization gzochid_channel_serialization;
 
-gzochid_channel *gzochid_channel_create 
-(gzochid_application_context *, char *name);
-gzochid_channel *gzochid_channel_get 
-(gzochid_application_context *, char *name);
+extern gzochid_application_task_serialization
+gzochid_channel_operation_task_serialization;
+
+/*
+  Create and return a new channel with the specified name in the specified game
+  application context.
+
+  Returns `NULL' if the transaction could not be created; e.g., if the current
+  transaction is marked for rollback.
+*/
+
+gzochid_channel *gzochid_channel_create (gzochid_application_context *, char *);
+gzochid_channel *gzochid_channel_get (gzochid_application_context *, char *);
+
+/*
+  Initialize and return a new channel with the specified name. 
+  
+  Note that this function does not enqueue the channel for persistence nor bind
+  it to a Scheme object. Call `gzochid_channel_create' to do those things.
+*/
+
 gzochid_channel *gzochid_channel_new (char *);
+
+/*
+  Frees the memory associated with the specified channel.
+
+  Note that this function does not enqueue the channel for persistence nor bind
+  it to a Scheme object. Call `gzochid_channel_create' to do those things.
+*/
+
 void gzochid_channel_free (gzochid_channel *);
 
 void gzochid_channel_join 
@@ -56,4 +70,17 @@ void gzochid_channel_send
 (gzochid_application_context *, gzochid_channel *,  unsigned char *, short);
 void gzochid_channel_close (gzochid_application_context *, gzochid_channel *);
 
+/* Returns the oid of the specified channel's Scheme representation. */
+
+guint64 gzochid_channel_scm_oid (gzochid_channel *);
+
+/*
+  Returns the specified channel's name.
+  
+  The returned pointer is owned by the channel and should not be modified or
+  freed by the caller. 
+*/   
+
+const char *gzochid_channel_name (gzochid_channel *);
+
 #endif /* GZOCHID_CHANNEL_H */
