@@ -1,5 +1,5 @@
 /* data.c: Application data management routines for gzochid
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
 
 #include <assert.h>
 #include <glib.h>
-#include <gmp.h>
 #include <stddef.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -27,18 +27,24 @@
 #include "game.h"
 #include "gzochid-storage.h"
 #include "io.h"
-#include "log.h"
+#include "oids.h"
 #include "tx.h"
+#include "util.h"
 
-#define ALLOCATION_BLOCK_SIZE 100
+struct _gzochid_data_oid_block_state
+{
+  guint64 first;
+  guint64 next;
+  guint64 last;  
+};
 
-#define NEXT_OID_KEY 0x00
+typedef struct _gzochid_data_oid_block_state gzochid_data_oid_block_state;
 
 struct _gzochid_data_transaction_context
 {
   gzochid_application_context *context;
 
-  gzochid_data_oid_block *free_oids;
+  gzochid_data_oid_block_state *free_oids;
   GList *used_oid_blocks;
 
   gzochid_storage_transaction *transaction;
@@ -61,17 +67,15 @@ gzochid_data_error_quark (void)
 gzochid_oid_holder *
 gzochid_oid_holder_new (void)
 {
-  gzochid_oid_holder *holder = malloc (sizeof (gzochid_oid_holder));
-
-  mpz_init (holder->oid);
-
-  return holder;
+  return calloc (1, sizeof (gzochid_oid_holder));
 }
 
 void 
 gzochid_oid_holder_free (gzochid_oid_holder *holder)
 {
-  mpz_clear (holder->oid);
+  if (holder->err != NULL)
+    g_error_free (holder->err);
+  
   free (holder);
 }
 
@@ -91,32 +95,33 @@ create_transaction_context (gzochid_application_context *app_context,
 	 ->transaction_begin (app_context->storage_context);
 
   tx_context->oids_to_references = g_hash_table_new_full 
-    (g_str_hash, g_str_equal, free, NULL);
+    (g_int64_hash, g_int64_equal, free, NULL);
   tx_context->ptrs_to_references = g_hash_table_new 
     (g_direct_hash, g_direct_equal);
 
   return tx_context;
 }
 
-static void 
-free_oid_block (void *ptr)
+static gzochid_data_oid_block_state *
+create_oid_block_state (gzochid_data_oids_block *block)
 {
-  gzochid_data_oid_block *block = ptr;
+  gzochid_data_oid_block_state *block_state =
+    malloc (sizeof (gzochid_data_oid_block_state));
 
-  mpz_clear (block->first);
-  mpz_clear (block->next);
-  mpz_clear (block->last);
-
-  free (block);
-} 
+  block_state->first = block->block_start;
+  block_state->next = block->block_start;
+  block_state->last = block_state->first + block->block_size - 1;
+  
+  return block_state;
+}
 
 static void 
 transaction_context_free (gzochid_data_transaction_context *context)
 {  
   if (context->free_oids != NULL)
-    free_oid_block (context->free_oids);
+    free (context->free_oids);
 
-  g_list_free_full (context->used_oid_blocks, free_oid_block);
+  g_list_free_full (context->used_oid_blocks, free);
 
   g_hash_table_destroy (context->oids_to_references);
   g_hash_table_destroy (context->ptrs_to_references);
@@ -128,10 +133,11 @@ static gboolean
 flush_reference (gzochid_data_managed_reference *reference,
 		 gzochid_data_transaction_context *context)
 {
-  char *oid_str = NULL;
-  GString *out = NULL;
+  GByteArray *out = NULL;
   GError *err = NULL;
 
+  guint64 encoded_oid = gzochid_util_encode_oid (reference->oid);
+
   switch (reference->state)
     {
     case GZOCHID_MANAGED_REFERENCE_STATE_EMPTY:
@@ -141,35 +147,36 @@ flush_reference (gzochid_data_managed_reference *reference,
     case GZOCHID_MANAGED_REFERENCE_STATE_NEW:
     case GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED:
       
-      out = g_string_new (NULL);
+      out = g_byte_array_new ();
       assert (reference->obj != NULL);
 
-      oid_str = mpz_get_str (NULL, 16, reference->oid);
-      gzochid_debug ("Flushing new/modified reference '%s'.", oid_str);
+      g_debug ("Flushing new/modified reference '%lx'.", reference->oid);
 
       reference->serialization->serializer
 	(context->context, reference->obj, out, &err);
 
       if (err != NULL || gzochid_transaction_rollback_only ())
 	{
-	  free (oid_str);
 	  g_clear_error (&err);
-	  g_string_free (out, TRUE);
+	  g_byte_array_unref (out);
 	  return FALSE;
 	}
 
       APP_STORAGE_INTERFACE (context->context)->transaction_put
-	(context->transaction, context->context->oids, oid_str, 
-	 strlen (oid_str) + 1, out->str, out->len);
-      free (oid_str);
+	(context->transaction, context->context->oids,
+	 (char *) &encoded_oid, sizeof (guint64),
+	 (char *) out->data, out->len);
 
       gzochid_application_event_dispatch
 	(context->context->event_source,
 	 gzochid_application_data_event_new (BYTES_WRITTEN, out->len));
 
-      g_string_free (out, TRUE);
-
-      break;
+      g_byte_array_unref (out);
+      
+      if (context->transaction->rollback)
+	return FALSE;
+      else break;
+      
     case GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_FETCHED:
     case GZOCHID_MANAGED_REFERENCE_STATE_NOT_MODIFIED: break;
     default:
@@ -189,11 +196,6 @@ data_prepare (gpointer data)
   gzochid_storage_engine_interface *iface = 
     APP_STORAGE_INTERFACE (context->context);
 
-  iface->lock (context->context->oids);
-  iface->lock (context->context->names);
-
-  context->needs_unlock = TRUE;
-  
   while (reference_ptr != NULL)
     {
       if (!flush_reference (reference_ptr->data, context))
@@ -216,15 +218,6 @@ data_prepare (gpointer data)
   return TRUE;
 }
 
-static void 
-free_reference (gpointer data)
-{
-  gzochid_data_managed_reference *reference = data;
-
-  mpz_clear (reference->oid);
-  free (data);
-}
-
 static void 
 finalize_references (gzochid_data_transaction_context *context)
 {
@@ -243,7 +236,8 @@ finalize_references (gzochid_data_transaction_context *context)
 	}
       reference_ptr = reference_ptr->next;
     }
-  g_list_free_full (references, free_reference);
+  
+  g_list_free_full (references, free);
 }
 
 static void 
@@ -254,12 +248,7 @@ data_commit (gpointer data)
     APP_STORAGE_INTERFACE (context->context);
 
   iface->transaction_commit (context->transaction);
-
   finalize_references (context);
-
-  iface->unlock (context->context->names);
-  iface->unlock (context->context->oids);
-
   transaction_context_free (context);
 }
 
@@ -278,15 +267,7 @@ data_rollback (gpointer data)
 	APP_STORAGE_INTERFACE (context->context);
 
       iface->transaction_rollback (context->transaction);
-
       finalize_references (context);
-
-      if (context->needs_unlock)
-	{
-	  iface->unlock (context->context->names);
-	  iface->unlock (context->context->oids);
-	}
-      
       transaction_context_free (context);
     }
 }
@@ -294,105 +275,83 @@ data_rollback (gpointer data)
 static gzochid_transaction_participant data_participant = 
   { "data", data_prepare, data_commit, data_rollback };
 
-static gzochid_data_oid_block *
-create_oid_block (gzochid_application_context *context)
-{
-  char next_oid_key[] = { NEXT_OID_KEY };
-  char *next_oid_value = NULL;
-  size_t next_oid_value_len = 0;
-
-  gzochid_data_oid_block *block = calloc (1, sizeof (gzochid_data_oid_block));
-  gzochid_storage_engine_interface *iface = APP_STORAGE_INTERFACE (context);
-
-  mpz_init (block->first);
-  mpz_init (block->next);
-  mpz_init (block->last);
-
-  iface->lock (context->meta);
-
-  next_oid_value = iface->get
-    (context->meta, next_oid_key, 1, &next_oid_value_len);
-
-  if (next_oid_value != NULL)
-    {
-      mpz_set_str (block->first, next_oid_value, 16);
-
-      mpz_set (block->next, block->first);
-      mpz_set (block->last, block->first);
-      free (next_oid_value);
-    }
-  
-  mpz_add_ui (block->last, block->last, ALLOCATION_BLOCK_SIZE);
-
-  next_oid_value = mpz_get_str (NULL, 16, block->last);
-  next_oid_value_len = strlen (next_oid_value) + 1;
-  iface->put 
-    (context->meta, next_oid_key, 1, next_oid_value, next_oid_value_len);
-  free (next_oid_value);
-
-  mpz_sub_ui (block->last, block->last, 1);
-  
-  iface->unlock (context->meta);
-
-  return block;
-}
-
-static gzochid_data_oid_block *
+static gzochid_data_oid_block_state *
 reserve_oids (gzochid_application_context *context)
 {
-  gzochid_data_oid_block *block = NULL;
+  gzochid_data_oid_block_state *block_state = NULL;
   
   g_mutex_lock (&context->free_oids_lock);
   
   if (g_list_length (context->free_oid_blocks) > 0)
     {
-      block = (gzochid_data_oid_block *) context->free_oid_blocks->data;
+      block_state = context->free_oid_blocks->data;
       context->free_oid_blocks = g_list_remove_link 
 	(context->free_oid_blocks, context->free_oid_blocks);
     }
-  else block = create_oid_block (context);
+  else
+    {
+      gzochid_data_oids_block new_block;
+
+      assert (gzochid_oids_reserve_block
+	      (context->oid_strategy, &new_block, NULL));
+
+      block_state = create_oid_block_state (&new_block);
+    }
 
   g_mutex_unlock (&context->free_oids_lock);
-  return block;
+  return block_state;
 }
 
-static void 
-get_binding (gzochid_data_transaction_context *context, char *name, mpz_t oid, 
+static guint64
+get_binding (gzochid_data_transaction_context *context, char *name,
 	     GError **err)
 {
-  char *oid_str = APP_STORAGE_INTERFACE (context->context)->transaction_get 
+  size_t oid_bytes_len = 0;
+  char *oid_bytes = APP_STORAGE_INTERFACE (context->context)->transaction_get 
     (context->transaction, context->context->names, name, strlen (name) + 1, 
-     NULL);
+     &oid_bytes_len);
 
+  assert (oid_bytes == NULL || oid_bytes_len == sizeof (guint64));
+  
   if (context->transaction->rollback)
     {
       gzochid_transaction_mark_for_rollback 
 	(&data_participant, context->transaction->should_retry);
+
       g_set_error
 	(err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION, 
 	 "Transaction rolled back while retrieving binding.");
+
+      return 0;
     }
-  else if (oid_str != NULL)
+  else if (oid_bytes != NULL)
     {
-      mpz_set_str (oid, oid_str, 16);
-      free (oid_str);
+      guint64 oid;
+      
+      memcpy (&oid, oid_bytes, sizeof (guint64));
+      free (oid_bytes);
+
+      return gzochid_util_decode_oid (oid);
     }
-  else g_set_error
+ else
+   {
+     g_set_error
 	(err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_NOT_FOUND, 
 	 "No data found for binding '%s'.", name);
+     
+     return 0;
+   }
 }
 
 static void 
-set_binding (gzochid_data_transaction_context *context, char *name, mpz_t oid, 
+set_binding (gzochid_data_transaction_context *context, char *name, guint64 oid,
 	     GError **err)
 {
-  char *oid_str = mpz_get_str (NULL, 16, oid);
-
+  guint64 encoded_oid = gzochid_util_encode_oid (oid);
+  
   APP_STORAGE_INTERFACE (context->context)->transaction_put 
     (context->transaction, context->context->names, name, strlen (name) + 1, 
-     oid_str, strlen (oid_str) + 1);
-
-  free (oid_str);
+     (char *) &encoded_oid, sizeof (guint64));
 
   if (context->transaction->rollback)
     {
@@ -405,17 +364,14 @@ set_binding (gzochid_data_transaction_context *context, char *name, mpz_t oid,
 }
 
 static gzochid_data_managed_reference *
-create_empty_reference (gzochid_application_context *context, mpz_t oid, 
+create_empty_reference (gzochid_application_context *context, guint64 oid, 
 			gzochid_io_serialization *serialization)
 {
   gzochid_data_managed_reference *reference = 
     calloc (1, sizeof (gzochid_data_managed_reference));
 
   reference->context = context;
-
-  mpz_init (reference->oid);
-  mpz_set (reference->oid, oid);
- 
+  reference->oid = oid;
   reference->state = GZOCHID_MANAGED_REFERENCE_STATE_EMPTY;
   reference->serialization = serialization;
   
@@ -423,15 +379,15 @@ create_empty_reference (gzochid_application_context *context, mpz_t oid,
 }
 
 static int 
-next_object_id (gzochid_data_transaction_context *context, mpz_t oid)
+next_object_id (gzochid_data_transaction_context *context, guint64 *oid)
 {
   if (context->free_oids == NULL)
     context->free_oids = reserve_oids (context->context);
 
-  mpz_set (oid, context->free_oids->next);
-  mpz_add_ui (context->free_oids->next, context->free_oids->next, 1);
+  *oid = context->free_oids->next;
+  context->free_oids->next += 1;
 
-  if (mpz_cmp (context->free_oids->next, context->free_oids->last) > 0)
+  if (context->free_oids->next > context->free_oids->last)
     {
       context->used_oid_blocks = g_list_append 
 	(context->used_oid_blocks, context->free_oids);
@@ -449,42 +405,59 @@ create_new_reference (gzochid_application_context *context, void *ptr,
     gzochid_transaction_context (&data_participant);
   gzochid_data_managed_reference *reference = 
     calloc (1, sizeof (gzochid_data_managed_reference));
-
+  
   reference->context = context;
   
-  mpz_init (reference->oid);
-  
-  next_object_id (tx_context, reference->oid);
+  next_object_id (tx_context, &reference->oid);
   reference->state = GZOCHID_MANAGED_REFERENCE_STATE_NEW;
   reference->serialization = serialization;
   reference->obj = ptr;
 
+  /*
+    Once this function returns, callers will have an oid of the object that will
+    be the permanent / effective oid if the current transaction commits. To
+    prevent a flurry of contention around the end of the transaction, take a
+    write lock on the oid now, first checking to make sure the underlying 
+    storage transaction is in a good state.
+  */
+  
+  if (!tx_context->transaction->rollback)
+    {
+      guint64 encoded_oid = gzochid_util_encode_oid (reference->oid);
+      
+      APP_STORAGE_INTERFACE (context)->transaction_get_for_update
+	(tx_context->transaction, tx_context->context->oids,
+	 (char *) &encoded_oid, sizeof (guint64), NULL);
+    }
+  
   return reference;
 }
 
 static gzochid_data_managed_reference *
-get_reference_by_oid (gzochid_application_context *context, mpz_t oid, 
+get_reference_by_oid (gzochid_application_context *context, guint64 oid, 
 		      gzochid_io_serialization *serialization)
 {
   gzochid_data_transaction_context *tx_context = 
     gzochid_transaction_context (&data_participant);
-  char *key = mpz_get_str (NULL, 16, oid);
   gzochid_data_managed_reference *reference = g_hash_table_lookup 
-    (tx_context->oids_to_references, key);
+    (tx_context->oids_to_references, &oid);
 
   if (reference == NULL)
     {
+      guint64 *key = malloc (sizeof (guint64));
+
+      *key = oid;
+      
       reference = create_empty_reference (context, oid, serialization);
       g_hash_table_insert (tx_context->oids_to_references, key, reference);
     }
-  else free (key);
 
   return reference;
 }
 
 static gzochid_data_managed_reference *
 get_reference_by_ptr (gzochid_application_context *context, void *ptr, 
-		      gzochid_io_serialization *serialization)
+		      gzochid_io_serialization *serialization, GError **err)
 {
   gzochid_data_transaction_context *tx_context = 
     gzochid_transaction_context (&data_participant);
@@ -493,18 +466,30 @@ get_reference_by_ptr (gzochid_application_context *context, void *ptr,
 
   if (reference == NULL)
     {
-      char *key = NULL;
-
+      guint64 *key = NULL;
+      
       reference = create_new_reference (context, ptr, serialization);
-      key = mpz_get_str (NULL, 16, reference->oid);
 
       assert (g_hash_table_lookup 
-	      (tx_context->oids_to_references, key) == NULL);
+	      (tx_context->oids_to_references, &reference->oid) == NULL);
       assert (g_hash_table_lookup 
 	      (tx_context->ptrs_to_references, ptr) == NULL);
+
+      key = malloc (sizeof (guint64));
+      *key = reference->oid;
       
       g_hash_table_insert (tx_context->oids_to_references, key, reference);
       g_hash_table_insert (tx_context->ptrs_to_references, ptr, reference);
+
+      if (tx_context->transaction->rollback)
+	{
+	  gzochid_transaction_mark_for_rollback 
+	    (&data_participant, tx_context->transaction->should_retry);
+	  g_set_error 
+	    (err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION, 
+	     "Transaction rolled back while creating new reference.");
+	  return NULL;
+	}
     }
   
   return reference;
@@ -512,23 +497,38 @@ get_reference_by_ptr (gzochid_application_context *context, void *ptr,
 
 static void 
 dereference (gzochid_data_transaction_context *context, 
-	     gzochid_data_managed_reference *reference, GError **err)
+	     gzochid_data_managed_reference *reference, gboolean for_update,
+	     GError **err)
 {
   size_t data_len = 0;
-  char *oid_str = NULL; 
   char *data = NULL; 
-  GString *in = NULL;
+  GByteArray *in = NULL;
+  guint64 encoded_oid;
   
-  if (reference->obj != NULL 
+  if (reference->obj != NULL
       || reference->state == GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_EMPTY)
-    return;
+    {
+      /* Does the lock need to be upgraded? */
+      
+      if (for_update &&
+	  reference->state == GZOCHID_MANAGED_REFERENCE_STATE_NOT_MODIFIED)
+	gzochid_data_mark
+	  (context->context, reference->serialization, reference->obj, err);
+      
+      return;
+    }
 
-  oid_str = mpz_get_str (NULL, 16, reference->oid);
-  gzochid_debug ("Retrieving data for reference '%s'.", oid_str);
+  g_debug ("Retrieving data for reference '%lx'.", reference->oid);
 
-  data = APP_STORAGE_INTERFACE (context->context)->transaction_get
-    (context->transaction, context->context->oids, oid_str, 
-     strlen (oid_str) + 1, &data_len);
+  encoded_oid = gzochid_util_encode_oid (reference->oid);
+  
+  if (for_update)
+    data = APP_STORAGE_INTERFACE (context->context)->transaction_get_for_update
+      (context->transaction, context->context->oids, (char *) &encoded_oid,
+       sizeof (guint64), &data_len);
+  else data = APP_STORAGE_INTERFACE (context->context)->transaction_get
+	 (context->transaction, context->context->oids, (char *) &encoded_oid, 
+	  sizeof (guint64), &data_len);
 
   if (context->transaction->rollback)
     {
@@ -536,26 +536,24 @@ dereference (gzochid_data_transaction_context *context,
 	(&data_participant, context->transaction->should_retry);
       g_set_error 
 	(err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION, 
-	 "Transaction rolled back while retrieving reference '%s'.", oid_str);
-      free (oid_str);
+	 "Transaction rolled back while retrieving reference '%lx'.",
+	 reference->oid);
     }
   else if (data == NULL)
-    {
-      g_set_error 
-	(err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_NOT_FOUND, 
-	 "No data found for reference '%s'.", oid_str);
-
-      free (oid_str);
-    }
+    g_set_error 
+      (err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_NOT_FOUND, 
+       "No data found for reference '%lx'.", reference->oid);
   else 
     {
       GError *local_err = NULL;
-
+      guint64 *key = malloc (sizeof (guint64));
+      
       gzochid_application_event_dispatch
 	(context->context->event_source,
 	 gzochid_application_data_event_new (BYTES_READ, data_len));
 
-      in = g_string_new_len (data, data_len);
+      in = g_byte_array_sized_new (data_len);
+      g_byte_array_append (in, (unsigned char *) data, data_len);
       free (data);
 
       reference->obj = reference->serialization->deserializer 
@@ -563,48 +561,52 @@ dereference (gzochid_data_transaction_context *context,
 
       if (local_err != NULL)
 	g_propagate_error (err, local_err);	  
+      else if (for_update)
+	reference->state = GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED;
       else reference->state = GZOCHID_MANAGED_REFERENCE_STATE_NOT_MODIFIED;
+
+      *key = reference->oid;
       
-      g_hash_table_insert (context->oids_to_references, oid_str, reference);
+      g_hash_table_insert (context->oids_to_references, key, reference);
       g_hash_table_insert 
 	(context->ptrs_to_references, reference->obj, reference);
 
-      g_string_free (in, TRUE);
+      g_byte_array_unref (in);
     }
 
   return;
 }
 
 static void 
-remove_object (gzochid_data_transaction_context *context, mpz_t oid, 
+remove_object (gzochid_data_transaction_context *context, guint64 oid, 
 	       GError **err)
 {
   int ret = 0;
-  char *oid_str = mpz_get_str (NULL, 16, oid);
 
   /* The transaction is presumed to have already been joined at this point. */
   
   gzochid_data_transaction_context *tx_context =
     gzochid_transaction_context (&data_participant);
 
-  gzochid_debug ("Removing reference '%s'.", oid_str);      
+  guint64 encoded_oid = gzochid_util_encode_oid (oid);
+  
+  g_debug ("Removing reference '%lx'.", oid);
   ret = APP_STORAGE_INTERFACE (context->context)->transaction_delete 
-    (context->transaction, context->context->oids, oid_str, 
-     strlen (oid_str) + 1);
+    (context->transaction, context->context->oids, (char *) &encoded_oid, 
+     sizeof (guint64));
 
   if (ret == GZOCHID_STORAGE_ENOTFOUND)
     g_set_error (err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_NOT_FOUND, 
-		 "No data found for reference '%s'.", oid_str);
+		 "No data found for reference '%lx'.", oid);
   else if (ret == GZOCHID_STORAGE_ETXFAILURE)
     {
       gzochid_transaction_mark_for_rollback 
 	(&data_participant, tx_context->transaction->should_retry);
       g_set_error 
 	(err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION, 
-	 "Transaction rolled back while removing reference '%s'.", oid_str);
+	 "Transaction rolled back while removing reference '%lx'.", oid);
     }
 
-  free (oid_str);
   return;
 }
 
@@ -667,9 +669,9 @@ void *
 gzochid_data_get_binding (gzochid_application_context *context, char *name, 
 			  gzochid_io_serialization *serialization, GError **err)
 {
-  mpz_t oid;
-  gzochid_data_managed_reference *reference = NULL;
+  guint64 oid;
   GError *local_err = NULL;
+  gzochid_data_managed_reference *reference = NULL;
   gzochid_data_transaction_context *tx_context = 
     join_transaction (context, &local_err);
   
@@ -679,26 +681,23 @@ gzochid_data_get_binding (gzochid_application_context *context, char *name,
       return NULL;
     }
 
-  mpz_init (oid);
-  get_binding (tx_context, name, oid, &local_err);
+  oid = get_binding (tx_context, name, &local_err);
 
   if (local_err != NULL)
     {
       g_propagate_error (err, local_err);
-      mpz_clear (oid);
       return NULL;
     }
 
   reference = get_reference_by_oid (context, oid, serialization);  
-  mpz_clear (oid);
 
-  dereference (tx_context, reference, err);
+  dereference (tx_context, reference, FALSE, err);
   return reference->obj;
 }
 
 void 
 gzochid_data_set_binding_to_oid (gzochid_application_context *context, 
-				 char *name, mpz_t oid, GError **err)
+				 char *name, guint64 oid, GError **err)
 {
   GError *local_err = NULL;
   gzochid_data_transaction_context *tx_context = 
@@ -711,7 +710,7 @@ gzochid_data_set_binding_to_oid (gzochid_application_context *context,
 
 char *
 gzochid_data_next_binding_oid (gzochid_application_context *context, char *key,
-			       mpz_t oid, GError **err)
+			       guint64 *oid, GError **err)
 {
   char *next_key = NULL;
   GError *local_err = NULL;
@@ -734,20 +733,17 @@ gzochid_data_next_binding_oid (gzochid_application_context *context, char *key,
 
   if (next_key != NULL)
     {
-      char *next_value = iface->transaction_get 
-	(tx_context->transaction, tx_context->context->names, next_key, 
-	 strlen (next_key) + 1, NULL);
-      if (tx_context->transaction->rollback)
-	gzochid_transaction_mark_for_rollback 
-	  (&data_participant, tx_context->transaction->should_retry);
+      GError *local_err = NULL;
+      guint64 local_oid = get_binding (tx_context, next_key, &local_err);
 
-      if (next_value != NULL)
+      if (local_err != NULL)
 	{
-	  mpz_set_str (oid, next_value, 16);
-	  free (next_value);
-	  return next_key;
+	  g_propagate_error (err, local_err);
+	  return NULL;
 	}
-      else return NULL;
+
+      *oid = local_oid;
+      return next_key;
     }
   else return NULL;
 }
@@ -766,8 +762,12 @@ gzochid_data_set_binding (gzochid_application_context *context, char *name,
     g_propagate_error (err, local_err);
   else 
     {
-      reference = get_reference_by_ptr (context, data, serialization);
-      set_binding (tx_context, name, reference->oid, err);
+      reference = get_reference_by_ptr
+	(context, data, serialization, &local_err);
+      if (local_err == NULL)
+	set_binding (tx_context, name, reference->oid, &local_err);
+      if (local_err != NULL)
+	g_propagate_error (err, local_err);
     }
 }
 
@@ -807,7 +807,6 @@ gboolean
 gzochid_data_binding_exists (gzochid_application_context *context, char *name, 
 			     GError **err)
 {
-  mpz_t oid;
   gboolean ret = TRUE;
   GError *local_err = NULL;
   gzochid_data_transaction_context *tx_context = 
@@ -819,10 +818,8 @@ gzochid_data_binding_exists (gzochid_application_context *context, char *name,
       return FALSE;
     }
 
-  mpz_init (oid);
-
-  get_binding (tx_context, name, oid, &local_err);
-
+  get_binding (tx_context, name, &local_err);
+  
   if (local_err != NULL)
     {
       if (local_err->code != GZOCHID_DATA_ERROR_NOT_FOUND)       
@@ -832,37 +829,28 @@ gzochid_data_binding_exists (gzochid_application_context *context, char *name,
       ret = FALSE;
     }
 
-  mpz_clear (oid);
   return ret;
 }
 
 gzochid_data_managed_reference *gzochid_data_create_reference
 (gzochid_application_context *context, 
- gzochid_io_serialization *serialization, void *ptr)
+ gzochid_io_serialization *serialization, void *ptr, GError **err)
 {
-  GError *err = NULL;
-  join_transaction (context, &err);
+  GError *local_err = NULL;
+  join_transaction (context, &local_err);
 
-  if (err != NULL)
+  if (local_err != NULL)
     {
-      mpz_t oid;
-      gzochid_data_managed_reference *ret = NULL;
-
-      g_error_free (err);
-      mpz_init (oid);      
-      ret = create_empty_reference (context, oid, serialization);
-      mpz_clear (oid);
-
-      return ret;
+      g_propagate_error (err, local_err);
+      return NULL;
     }
-
-  return get_reference_by_ptr (context, ptr, serialization);
+  return get_reference_by_ptr (context, ptr, serialization, err);
 }
 
 gzochid_data_managed_reference *
 gzochid_data_create_reference_to_oid (gzochid_application_context *context, 
 				      gzochid_io_serialization *serialization, 
-				      mpz_t oid)
+				      guint64 oid)
 {
   GError *err = NULL;
 
@@ -876,7 +864,7 @@ gzochid_data_create_reference_to_oid (gzochid_application_context *context,
   else return get_reference_by_oid (context, oid, serialization);
 }
 
-void 
+void *
 gzochid_data_dereference (gzochid_data_managed_reference *reference, 
 			  GError **err)
 {
@@ -886,7 +874,24 @@ gzochid_data_dereference (gzochid_data_managed_reference *reference,
   
   if (local_err != NULL)
     g_propagate_error (err, local_err);
-  else dereference (tx_context, reference, err);
+  else dereference (tx_context, reference, FALSE, err);
+
+  return reference->obj;
+}
+
+void *
+gzochid_data_dereference_for_update (gzochid_data_managed_reference *reference, 
+				     GError **err)
+{
+  GError *local_err = NULL;
+  gzochid_data_transaction_context *tx_context = 
+    join_transaction (reference->context, &local_err);
+  
+  if (local_err != NULL)
+    g_propagate_error (err, local_err);
+  else dereference (tx_context, reference, TRUE, err);
+
+  return reference->obj;
 }
 
 void 
@@ -918,20 +923,21 @@ gzochid_data_mark (gzochid_application_context *context,
       return;
     }
 
-  reference = get_reference_by_ptr (context, ptr, serialization);
+  reference = get_reference_by_ptr (context, ptr, serialization, err);
 
-  if (reference->state != GZOCHID_MANAGED_REFERENCE_STATE_NEW
+  if (reference != NULL
+      && reference->state != GZOCHID_MANAGED_REFERENCE_STATE_NEW
       && reference->state != GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED)
     {
       char *data = NULL;
-      char *oid_str = mpz_get_str (NULL, 16, reference->oid); 
-
+      guint64 encoded_oid = gzochid_util_encode_oid (reference->oid);
+      
       reference->state = GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED;
-      gzochid_debug ("Marking reference '%s' for update.", oid_str);
+      g_debug ("Marking reference '%lx' for update.", reference->oid);
 
       data = APP_STORAGE_INTERFACE (context)->transaction_get_for_update
-	(tx_context->transaction, tx_context->context->oids, oid_str, 
-	 strlen (oid_str) + 1, NULL);
+	(tx_context->transaction, tx_context->context->oids,
+	 (char *) &encoded_oid, sizeof (guint64), NULL);
    
       if (tx_context->transaction->rollback)
 	{
@@ -939,11 +945,56 @@ gzochid_data_mark (gzochid_application_context *context,
 	    (&data_participant, tx_context->transaction->should_retry);
 	  g_set_error 
 	    (err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION, 
-	     "Transaction rolled back while removing reference '%s'.", 
-	     oid_str);
+	     "Transaction rolled back while removing reference '%lx'.",
+	     reference->oid);
 	}
       else free (data);
-
-      free (oid_str);
     }
 }
+
+/*
+  Print to standard output the list of managed references accessed by the
+  specified transaction context, along with their state - `MODIFIED', `NEW', 
+  etc.
+
+  Use this function within GDB to inspect the behavior of transactions against
+  the data service.
+*/
+
+void
+gzochid_data_print_references (gzochid_data_transaction_context *context)
+{
+  GList *values = g_hash_table_get_values (context->ptrs_to_references);
+  GList *values_ptr = values;
+
+  while (values_ptr != NULL)
+    {
+      const char *state;
+      gzochid_data_managed_reference *ref = values_ptr->data;
+      
+      switch (ref->state)
+	{
+	case GZOCHID_MANAGED_REFERENCE_STATE_NEW: state = "NEW"; break;
+	case GZOCHID_MANAGED_REFERENCE_STATE_EMPTY: state = "EMPTY"; break;
+	case GZOCHID_MANAGED_REFERENCE_STATE_NOT_MODIFIED:
+	  state = "NOT MODIFIED"; break;
+	case GZOCHID_MANAGED_REFERENCE_STATE_MAYBE_MODIFIED:
+	  state = "MAYBE MODIFIED"; break;
+	case GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED: state =
+	    "MODIFIED"; break;
+	case GZOCHID_MANAGED_REFERENCE_STATE_FLUSHED: state = "FLUSHED"; break;
+	case GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_EMPTY:
+	  state = "REMOVED EMPTY"; break;
+	case GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_FETCHED:
+	  state = "REMOVED FETCHED"; break;
+	default: state = "UNKNOWN";
+	}
+      
+      printf ("%lx (serialization: %p) -> %s\n", ref->oid, ref->serialization,
+	      state);
+
+      values_ptr = values_ptr->next;
+    }
+  
+  g_list_free (values);
+}
