@@ -1,5 +1,5 @@
 /* descriptor.c: Game application descriptor parsing routines for gzochid
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 #include <unistd.h>
 
 #include "descriptor.h"
-#include "log.h"
 
 struct _descriptor_builder_context
 {
@@ -154,25 +153,18 @@ descriptor_start_element (GMarkupParseContext *context,
 	    ("module", attribute_names, attribute_values);
 	  GList *module_name = to_module_name (module_name_str);
 
-	  mpz_t scm_oid;
-	  
-	  mpz_init (scm_oid);
-	  mpz_set_si (scm_oid, -1);
-
 	  if (strcmp (parent, "initialized") == 0)
 	    descriptor->initialized = gzochid_application_callback_new 
-	      (strdup (procedure), module_name, scm_oid);
+	      (strdup (procedure), module_name, 0);
 	  else if (strcmp (parent, "logged-in") == 0)
 	    descriptor->logged_in = gzochid_application_callback_new 
-	      (strdup (procedure), module_name, scm_oid);
+	      (strdup (procedure), module_name, 0);
 	  else if (strcmp (parent, "ready") == 0)
 	    descriptor->ready = gzochid_application_callback_new
-	      (strdup (procedure), module_name, scm_oid);
+	      (strdup (procedure), module_name, 0);
 	  else *error = g_error_new 
 		 (G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT, 
 		  "Invalid position for 'callback' element.");
-
-	  mpz_clear (scm_oid);
 	}
       else;
     }
@@ -312,7 +304,7 @@ gzochid_config_parse_application_descriptor (FILE *file)
 
   if (err != NULL)
     {
-      gzochid_warning (err->message);      
+      g_warning ("%s", err->message);      
       g_error_free (err);
       
       descriptor_free (descriptor);
