@@ -1,5 +1,5 @@
 /* game.c: Game context management routines for gzochid
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -26,19 +26,22 @@
 
 #include "app.h"
 #include "auth_int.h"
+#include "channel.h"
 #include "config.h"
 #include "context.h"
 #include "descriptor.h"
 #include "durable-task.h"
 #include "fsm.h"
 #include "game.h"
+#include "game-protocol.h"
 #include "gzochid.h"
 #include "gzochid-storage.h"
-#include "log.h"
+#include "resolver.h"
 #include "scheme.h"
 #include "scheme-task.h"
 #include "socket.h"
 #include "storage.h"
+#include "storage-dataclient.h"
 #include "storage-mem.h"
 #include "task.h"
 #include "threads.h"
@@ -85,7 +88,7 @@ scan_app_dir (gzochid_game_context *context, const char *dir)
 
   if (!g_file_test (descriptor_filename, G_FILE_TEST_IS_REGULAR))
     {
-      gzochid_warning 
+      g_warning 
 	("%s does not exist or is not a regular file.", descriptor_filename);
       g_free (descriptor_filename);
       return;
@@ -96,13 +99,13 @@ scan_app_dir (gzochid_game_context *context, const char *dir)
   fclose (descriptor_file);
 
   if (descriptor == NULL)
-    gzochid_warning
+    g_warning
       ("Failed to parse application descriptor %s; skipping.",
        descriptor_filename);
   else if (g_hash_table_contains (context->applications, descriptor->name))
-    gzochid_warning
+    g_warning
       ("Application in %s with name '%s' already exists; skipping.", 
-       descriptor_file, descriptor->name);
+       descriptor_filename, descriptor->name);
   else initialize_application (context, dir, descriptor);
 
   g_free (descriptor_filename);
@@ -115,9 +118,8 @@ qualify_apps_dir (gzochid_game_context *context)
     return g_strdup (context->apps_dir);
   else
     {
-      gzochid_server_context *server_context =
-	(gzochid_server_context *) ((gzochid_context *) context)->parent;
-      gchar *basedir = g_path_get_dirname (server_context->gzochid_conf_path);
+      gchar *basedir = g_path_get_dirname
+	(context->root_context->gzochid_conf_path);
       gchar *path = g_strconcat (basedir, "/", context->apps_dir, NULL);
       
       g_free (basedir);
@@ -155,13 +157,15 @@ initialize_auth (int from_state, int to_state, gpointer user_data)
 static void 
 initialize_server (int from_state, int to_state, gpointer user_data)
 {
-  gzochid_game_context *context = user_data;
+  gzochid_game_context *game_context = user_data;
 
-  context->server = gzochid_socket_server_context_new ();
-  gzochid_socket_server_context_init 
-    (context->server, (gzochid_context *) context, context->port);
-  gzochid_context_until 
-    ((gzochid_context *) context->server, GZOCHID_SOCKET_SERVER_STATE_RUNNING);
+  game_context->server_socket = gzochid_server_socket_new
+    ("Game server", gzochid_game_server_protocol, game_context);
+  gzochid_server_socket_listen
+    (game_context->socket_server, game_context->server_socket,
+     game_context->port);
+
+  gzochid_socket_server_start (game_context->socket_server);
 }
 
 static void 
@@ -171,31 +175,31 @@ initialize_apps (int from_state, int to_state, gpointer user_data)
   
   if (!g_file_test (context->work_dir, G_FILE_TEST_EXISTS))
     {
-      gzochid_notice 
+      g_message 
 	("Work directory %s does not exist; creating...", 
 	 context->work_dir);
       if (g_mkdir_with_parents (context->work_dir, 493) != 0)
 	{
-	  gzochid_err
+	  g_critical
 	    ("Unable to create work directory %s.", context->work_dir);
 	  exit (EXIT_FAILURE);
 	}
     }
   else if (!g_file_test (context->work_dir, G_FILE_TEST_IS_DIR))
     {
-      gzochid_err ("%s is not a directory.", context->work_dir);
+      g_critical ("%s is not a directory.", context->work_dir);
       exit (EXIT_FAILURE);
     }
   
   if (!g_file_test (context->apps_dir, G_FILE_TEST_EXISTS))
     {
-      gzochid_err 
+      g_critical 
 	("Application directory %s does not exist.", context->apps_dir);
       exit (EXIT_FAILURE);
     }
   else if (!g_file_test (context->apps_dir, G_FILE_TEST_IS_DIR))
     {
-      gzochid_err ("%s is not a directory.", context->apps_dir);
+      g_critical ("%s is not a directory.", context->apps_dir);
       exit (EXIT_FAILURE);
     }
 
@@ -210,25 +214,18 @@ initialize_scheme_api (int from_state, int to_state, gpointer user_data)
 }
 
 static void 
-initialize_serialization_registry (int from_state, int to_state, 
-				   gpointer user_data)
+initialize_application_task_serializations (int from_state, int to_state, 
+					    gpointer user_data)
 {
   gzochid_task_initialize_serialization_registry ();
-}
 
-static void 
-initialize_scheme_task_serialization (int from_state, int to_state,
-				      gpointer user_data)
-{
   gzochid_task_register_serialization (&gzochid_scheme_task_serialization);
-}
-
-static void 
-initialize_client_received_message_task_serialization (int from_state, 
-						       int to_state, 
-						       gpointer user_data)
-{
-  gzochid_register_client_received_message_task_serialization ();
+  gzochid_task_register_serialization
+    (&gzochid_client_received_message_task_serialization);
+  gzochid_task_register_serialization
+    (&gzochid_channel_operation_task_serialization);
+  gzochid_task_register_serialization
+    (&gzochid_task_chain_bootstrap_task_serialization);
 }
 
 static void 
@@ -253,13 +250,16 @@ initialize_event_loop (int from_state, int to_state, gpointer user_data)
 }
 
 gzochid_game_context *
-gzochid_game_context_new (void)
+gzochid_game_context_new ()
 {
+  GMainContext *event_context = g_main_context_new ();
   gzochid_game_context *context = calloc (1, sizeof (gzochid_game_context));
 
   context->applications = g_hash_table_new (g_str_hash, g_str_equal);
   context->auth_plugins = g_hash_table_new (g_str_hash, g_str_equal);
 
+  context->event_loop = g_main_loop_new (event_context, FALSE);
+
   return context;
 }
 
@@ -271,30 +271,38 @@ gzochid_game_context_free (gzochid_game_context *context)
   g_hash_table_destroy (context->applications);
   g_hash_table_destroy (context->auth_plugins);
 
+  if (context->root_context != NULL)
+    g_object_unref (context->root_context);
+  if (context->socket_server != NULL)
+    g_object_unref (context->socket_server);
+  
   free (context);
 } 
 
 void 
-gzochid_game_context_init (gzochid_game_context *context, 
-			   gzochid_context *parent, GHashTable *config)
+gzochid_game_context_init (gzochid_game_context *context,
+			   GzochidRootContext *root_context)
 {
   long tx_timeout_ms = 0;
   gzochid_fsm *fsm = gzochid_fsm_new 
     ("game", GZOCHID_GAME_STATE_INITIALIZING, "INITIALIZING");
-  GMainContext *event_context = g_main_context_new ();
-
-  gzochid_fsm_add_state (fsm, GZOCHID_GAME_STATE_RUNNING, "RUNNING");
-  gzochid_fsm_add_state (fsm, GZOCHID_GAME_STATE_STOPPED, "STOPPED");
-
-  context->event_loop = g_main_loop_new (event_context, FALSE);
+  GHashTable *config = gzochid_configuration_extract_group
+    (root_context->configuration, "game");
+  
+  context->root_context = g_object_ref (root_context);
+  context->socket_server = gzochid_resolver_require_full
+    (root_context->resolution_context, GZOCHID_TYPE_SOCKET_SERVER, NULL);
 
   context->pool = gzochid_thread_pool_new 
     (context, 
      gzochid_config_to_int 
      (g_hash_table_lookup (config, "thread_pool.max_threads"), 4), 
      TRUE, NULL);
-  context->task_queue = gzochid_schedule_task_queue_new (context->pool);
+  
+  gzochid_fsm_add_state (fsm, GZOCHID_GAME_STATE_RUNNING, "RUNNING");
+  gzochid_fsm_add_state (fsm, GZOCHID_GAME_STATE_STOPPED, "STOPPED");
 
+  context->task_queue = gzochid_schedule_task_queue_new (context->pool);
   gzochid_schedule_task_queue_start (context->task_queue);
 
   context->port = gzochid_config_to_int
@@ -313,7 +321,25 @@ gzochid_game_context_init (gzochid_game_context *context,
       strdup (g_hash_table_lookup (config, "auth.plugin.dir"));
   else context->auth_plugin_dir = GZOCHID_AUTH_PLUGIN_DIR;
 
-  if (g_hash_table_contains (config, "storage.engine"))
+  if (context->root_context->data_client != NULL)
+    {
+      char *conf_storage_engine = g_hash_table_lookup
+	(config, "storage.engine");
+
+      if (conf_storage_engine != NULL)
+	g_info
+	  ("Meta server client configuration detected; ignoring configured "
+	   "storage engine '%s'.", conf_storage_engine);
+      else g_info
+	     ("Meta server client configuration detected; using data client "
+	      "storage engine.");
+
+      context->storage_engine = calloc (1, sizeof (gzochid_storage_engine));
+
+      context->storage_engine->interface = 
+	&gzochid_storage_engine_interface_dataclient;
+    }
+  else if (g_hash_table_contains (config, "storage.engine"))
     {
       char *dir = NULL;
       char *env = getenv ("GZOCHID_STORAGE_ENGINE_DIR");
@@ -330,14 +356,14 @@ gzochid_game_context_init (gzochid_game_context *context,
       context->storage_engine = gzochid_storage_load_engine 
 	(dir, g_hash_table_lookup (config, "storage.engine"));
     }
-  else gzochid_info 
+  else g_info 
 	 ("No durable storage engine configured; memory engine will be used.");
 
   if (context->storage_engine == NULL)
     {
-      gzochid_info ("\
-Using in-memory storage for application data. THIS CONFIGURATION IS NOT SAFE \
-FOR PRODUCTION USE.");
+      g_info
+	("Using in-memory storage for application data. THIS CONFIGURATION IS "
+	 "NOT SAFE FOR PRODUCTION USE.");
 
       context->storage_engine = calloc (1, sizeof (gzochid_storage_engine));
       context->storage_engine->interface = 
@@ -358,17 +384,11 @@ FOR PRODUCTION USE.");
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_event_loop, context);
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_server, context);
-  gzochid_fsm_on_enter 
-    (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_serialization_registry, 
-     context);
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_scheme_api, context);
   gzochid_fsm_on_enter 
-    (fsm, GZOCHID_GAME_STATE_INITIALIZING, 
-     initialize_client_received_message_task_serialization, context);
-  gzochid_fsm_on_enter 
-    (fsm, GZOCHID_GAME_STATE_INITIALIZING, 
-     initialize_scheme_task_serialization, context);
+    (fsm, GZOCHID_GAME_STATE_INITIALIZING,
+     initialize_application_task_serializations, context);
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_auth, context);
   gzochid_fsm_on_enter 
@@ -376,7 +396,9 @@ FOR PRODUCTION USE.");
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_complete, context);
   
-  gzochid_context_init ((gzochid_context *) context, parent, fsm);
+  gzochid_context_init ((gzochid_context *) context, NULL, fsm);
+
+  g_hash_table_destroy (config);
 }
 
 void 
