@@ -1,5 +1,5 @@
 /* lifecycle.c: Handlers for gzochi application lifecycle events
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -24,13 +24,15 @@
 #include "auth_int.h"
 #include "fsm.h"
 #include "game.h"
+#include "game-protocol.h"
 #include "guile.h"
 #include "gzochid-auth.h"
 #include "lifecycle.h"
-#include "log.h"
-#include "protocol.h"
+#include "oids-dataclient.h"
+#include "oids-storage.h"
 #include "scheme-task.h"
 #include "session.h"
+#include "storage-dataclient.h"
 #include "task.h"
 
 G_LOCK_DEFINE_STATIC (load_path);
@@ -54,15 +56,15 @@ initialize_auth (int from_state, int to_state, gpointer user_data)
 	  gzochid_auth_plugin *plugin = g_hash_table_lookup 
 	    (game_context->auth_plugins, app_context->descriptor->auth_type);
 
-	  gzochid_debug 
-	    ("Initializing auth plugin '%s' for application '%'.",
+	  g_debug 
+	    ("Initializing auth plugin '%s' for application '%s'.",
 	     app_context->descriptor->auth_type,
 	     app_context->descriptor->name);
 
 	  app_context->auth_data = plugin->info->initialize 
 	    (app_context->descriptor->auth_properties, &error);
 	  if (error != NULL)
-	    gzochid_err
+	    g_critical
 	      ("Auth plugin '%s' failed to initialize "
 	       "for application '%s': %s",
 	       app_context->descriptor->auth_type,
@@ -72,14 +74,14 @@ initialize_auth (int from_state, int to_state, gpointer user_data)
 	  g_clear_error (&error);
 	  app_context->authenticator = plugin->info->authenticate;
 	}
-      else gzochid_err 
+      else g_critical 
 	     ("Unknown auth type '%s' for application '%s'.", 
 	      app_context->descriptor->auth_type,
 	      app_context->descriptor->name);
     }
   else
     {
-      gzochid_info 
+      g_info 
 	("No auth plugin specified for application '%s'; " 
 	 "pass-thru authentication will be used.", 
 	 app_context->descriptor->name);
@@ -104,30 +106,40 @@ initialize_data (int from_state, int to_state, gpointer user_data)
   gzochid_storage_context *storage_context = NULL;
   gzochid_storage_engine_interface *iface = APP_STORAGE_INTERFACE (app_context);
 
-  if (!g_file_test (data_dir, G_FILE_TEST_EXISTS))
+  if (game_context->storage_engine->handle != NULL)
     {
-      gzochid_notice 
-	("Work directory %s does not exist; creating...", data_dir);
-      if (g_mkdir_with_parents (data_dir, 493) != 0)
+      if (!g_file_test (data_dir, G_FILE_TEST_EXISTS))
 	{
-	  gzochid_err ("Unable to create work directory %s.", data_dir);
+	  g_message 
+	    ("Work directory %s does not exist; creating...", data_dir);
+	  if (g_mkdir_with_parents (data_dir, 493) != 0)
+	    {
+	      g_critical ("Unable to create work directory %s.", data_dir);
+	      exit (EXIT_FAILURE);
+	    }
+	}
+      else if (!g_file_test (data_dir, G_FILE_TEST_IS_DIR))
+	{
+	  g_critical ("%s is not a directory.", data_dir);
 	  exit (EXIT_FAILURE);
 	}
     }
-  else if (!g_file_test (data_dir, G_FILE_TEST_IS_DIR))
-    {
-      gzochid_err ("%s is not a directory.", data_dir);
-      exit (EXIT_FAILURE);
-    }
-
+  
   storage_context = iface->initialize (data_dir);
 
   if (storage_context == NULL)
     {
-      gzochid_err ("Unable to initialize storage. Exiting.");
+      g_critical ("Unable to initialize storage. Exiting.");
       exit (EXIT_FAILURE);
     }
 
+  if (iface == &gzochid_storage_engine_interface_dataclient)
+    {
+      assert (game_context->root_context->data_client != NULL);
+      gzochid_dataclient_storage_context_set_dataclient
+	(storage_context, game_context->root_context->data_client);
+    }
+  
   app_context->storage_context = storage_context;
   app_context->meta = iface->open 
     (storage_context, meta_db, GZOCHID_STORAGE_CREATE);
@@ -136,6 +148,15 @@ initialize_data (int from_state, int to_state, gpointer user_data)
   app_context->names = iface->open 
     (storage_context, names_db, GZOCHID_STORAGE_CREATE);
 
+  /* If we're using the dataclient storage engine, we need to use the dataclient
+     oid allocation strategy. */
+  
+  if (iface == &gzochid_storage_engine_interface_dataclient)
+    app_context->oid_strategy = gzochid_dataclient_oid_strategy_new
+      (game_context->root_context->data_client, app_context->descriptor->name);
+  else app_context->oid_strategy = gzochid_storage_oid_strategy_new
+	 (iface, app_context->storage_context, app_context->meta);
+
   free (data_dir);
   free (meta_db);
   free (oids_db);
@@ -216,7 +237,7 @@ initialize_async_transactional (gpointer data)
 
       if (err != NULL)
 	{
-	  gzochid_warning
+	  g_warning
 	    ("Failed to sweep sessions for application '%s'; stopping: %s.",
 	     context->descriptor->name, err->message);
 	  
@@ -353,54 +374,65 @@ static void
 login_catch_worker (gzochid_application_context *context,
 		    gzochid_auth_identity *identity, gpointer data)
 {
-  char *session_oid_str = data;
-  gzochid_protocol_client *client =
-    g_hash_table_lookup (context->oids_to_clients, session_oid_str);
+  guint64 *session_oid = data;
+  gzochid_game_client *client = g_hash_table_lookup
+    (context->oids_to_clients, session_oid);
 
-  gzochid_info
-    ("Disconnecting session '%s'; failed login transaction.", session_oid_str);
+  g_info
+    ("Disconnecting session '%lx'; failed login transaction.", *session_oid);
 
   g_mutex_lock (&context->client_mapping_lock);
-  g_hash_table_remove (context->oids_to_clients, session_oid_str);
+  g_hash_table_remove (context->oids_to_clients, session_oid);
   g_hash_table_remove (context->clients_to_oids, client);
   g_mutex_unlock (&context->client_mapping_lock);
       
-  gzochid_protocol_client_disconnect (client);
+  gzochid_game_client_disconnect (client);
 
-  gzochid_client_session_disconnected_worker
-    (context, identity, session_oid_str);
+  gzochid_client_session_disconnected_worker (context, identity, session_oid);
 }
 
-void 
-gzochid_application_client_logged_in (gzochid_application_context *context, 
-				      gzochid_protocol_client *client)
+/* The application task worker for the login event.  */
+
+static void
+logged_in_task (gzochid_application_context *context,
+		gzochid_auth_identity *identity, gpointer data)
 {
-  gzochid_auth_identity *identity =
-    gzochid_protocol_client_get_identity (client);
+  GError *err = NULL;
+  gzochid_game_client *client = data;
   gzochid_game_context *game_context = 
     (gzochid_game_context *) ((gzochid_context *) context)->parent;
   gzochid_client_session *session = gzochid_client_session_new (identity);
+  
+  guint64 *session_oid = malloc (sizeof (guint64));
 
+  gzochid_application_task *login_task = NULL;
+  gzochid_application_task *login_catch_task = NULL;
   gzochid_application_task *application_task = NULL;
-  gzochid_application_task *login_task = NULL, *login_catch_task = NULL;
-  gzochid_transactional_application_task_execution *execution = NULL;
+  gzochid_transactional_application_task_execution *execution = NULL;    
 
   gzochid_task task;
-
-  char *session_oid_str = NULL;
-  mpz_t session_oid;
   
-  mpz_init (session_oid);
-  gzochid_client_session_persist (context, session, session_oid);
-  session_oid_str = mpz_get_str (NULL, 16, session_oid);
-  mpz_clear (session_oid);
+  gzochid_client_session_persist (context, session, session_oid, &err);
+
+  if (err != NULL)
+    {
+      g_warning
+	("Failed to bind session oid for identity '%s'; disconnecting: %s",
+	 gzochid_auth_identity_name (identity), err->message);
+
+      g_error_free (err);
+      free (session_oid);
+      
+      gzochid_game_client_disconnect (client);
+      return;
+    }
 
   login_task = gzochid_application_task_new
     (context, identity, gzochid_scheme_application_logged_in_worker,
-     session_oid_str);
+     session_oid);
 
   login_catch_task = gzochid_application_task_new
-    (context, identity, login_catch_worker, session_oid_str);
+    (context, identity, login_catch_worker, session_oid);
 
   execution = gzochid_transactional_application_task_timed_execution_new 
     (login_task, login_catch_task, NULL, game_context->tx_timeout);
@@ -412,12 +444,12 @@ gzochid_application_client_logged_in (gzochid_application_context *context,
   gzochid_application_task_unref (login_catch_task);
 
   g_mutex_lock (&context->client_mapping_lock);
-  g_hash_table_insert (context->oids_to_clients, session_oid_str, client);
-  g_hash_table_insert (context->clients_to_oids, client, session_oid_str);
+  g_hash_table_insert (context->oids_to_clients, session_oid, client);
+  g_hash_table_insert (context->clients_to_oids, client, session_oid);
   g_mutex_unlock (&context->client_mapping_lock);
 
   application_task = gzochid_application_task_new
-    (context, gzochid_protocol_client_get_identity (client), 
+    (context, gzochid_game_client_get_identity (client), 
      gzochid_application_reexecuting_transactional_task_worker, execution);
      
   task.worker = gzochid_application_task_thread_worker;
@@ -428,16 +460,35 @@ gzochid_application_client_logged_in (gzochid_application_context *context,
 }
 
 void 
-gzochid_application_client_disconnected (gzochid_application_context *context, 
-					 gzochid_protocol_client *client)
+gzochid_application_client_logged_in (gzochid_application_context *context, 
+				      gzochid_game_client *client)
 {
   gzochid_game_context *game_context = 
     (gzochid_game_context *) ((gzochid_context *) context)->parent;
-  char *session_oid_str = NULL;
+  
+  gzochid_application_task *application_task = gzochid_application_task_new
+    (context, gzochid_game_client_get_identity (client), logged_in_task,
+     client);
+
+  gzochid_task *task = gzochid_task_immediate_new
+    (gzochid_application_task_thread_worker, application_task);
+
+  gzochid_schedule_submit_task (game_context->task_queue, task);
+
+  gzochid_task_free (task);
+}
+
+void 
+gzochid_application_client_disconnected (gzochid_application_context *context, 
+					 gzochid_game_client *client)
+{
+  gzochid_game_context *game_context = 
+    (gzochid_game_context *) ((gzochid_context *) context)->parent;
+  guint64 *session_oid = NULL;
 
   g_mutex_lock (&context->client_mapping_lock);
-  session_oid_str = g_hash_table_lookup (context->clients_to_oids, client);
-  if (session_oid_str == NULL)
+  session_oid = g_hash_table_lookup (context->clients_to_oids, client);
+  if (session_oid == NULL)
     {
       g_mutex_unlock (&context->client_mapping_lock);
       return;
@@ -446,22 +497,22 @@ gzochid_application_client_disconnected (gzochid_application_context *context,
     {
       gzochid_application_task *callback_task = 
 	gzochid_application_task_new
-	(context, gzochid_protocol_client_get_identity (client),
-	 gzochid_scheme_application_disconnected_worker, session_oid_str);
+	(context, gzochid_game_client_get_identity (client),
+	 gzochid_scheme_application_disconnected_worker, session_oid);
       gzochid_application_task *catch_task =
 	gzochid_application_task_new
-	(context, gzochid_protocol_client_get_identity (client),
-	 gzochid_client_session_disconnected_worker, session_oid_str);
+	(context, gzochid_game_client_get_identity (client),
+	 gzochid_client_session_disconnected_worker, session_oid);
       gzochid_application_task *cleanup_task = 
 	gzochid_application_task_new
-	(context, gzochid_protocol_client_get_identity (client), 
+	(context, gzochid_game_client_get_identity (client), 
 	 gzochid_scheme_application_disconnected_cleanup_worker,
-	 session_oid_str);
+	 session_oid);
       gzochid_transactional_application_task_execution *execution = 
-	gzochid_transactional_application_task_execution_new 
-	(callback_task, catch_task, cleanup_task);
+	gzochid_transactional_application_task_timed_execution_new 
+	(callback_task, catch_task, cleanup_task, game_context->tx_timeout);
       gzochid_application_task *application_task = gzochid_application_task_new 
-	(context, gzochid_protocol_client_get_identity (client),
+	(context, gzochid_game_client_get_identity (client),
 	 gzochid_application_resubmitting_transactional_task_worker, execution);
 
       gzochid_task task;
@@ -478,7 +529,7 @@ gzochid_application_client_disconnected (gzochid_application_context *context,
       gettimeofday (&task.target_execution_time, NULL);
 
       g_hash_table_remove (context->clients_to_oids, client);
-      g_hash_table_remove (context->oids_to_clients, session_oid_str);
+      g_hash_table_remove (context->oids_to_clients, session_oid);
       
       gzochid_schedule_submit_task (game_context->task_queue, &task);
 
@@ -486,54 +537,76 @@ gzochid_application_client_disconnected (gzochid_application_context *context,
     }
 }
 
+/* Cleanup handler for the received message event. */
+
+static void
+cleanup_received_message_arguments (gzochid_application_context *context,
+				    gzochid_auth_identity *identity,
+				    gpointer data)
+{
+  void **args = data;
+
+  /* The session oid (arg 0) comes from the client-session mapping table and
+     can't be freed here. */
+  
+  g_bytes_unref (args[1]);  
+  free (args);
+}
+
 void 
 gzochid_application_session_received_message
-(gzochid_application_context *context, gzochid_protocol_client *client, 
+(gzochid_application_context *context, gzochid_game_client *client, 
  unsigned char *msg, short len)
 {
   gzochid_game_context *game_context = 
     (gzochid_game_context *) ((gzochid_context *) context)->parent;
 
-  char *session_oid_str = NULL;
-  void *data[3];
+  guint64 *session_oid = NULL;
 
   g_mutex_lock (&context->client_mapping_lock);
-  session_oid_str = g_hash_table_lookup (context->clients_to_oids, client);
+  session_oid = g_hash_table_lookup (context->clients_to_oids, client);
   g_mutex_unlock (&context->client_mapping_lock);
 
-  if (session_oid_str == NULL)
+  if (session_oid == NULL)
     return;
   else 
     {
+      void **data = malloc (sizeof (void *) * 2);
+      
       gzochid_application_task *transactional_task = NULL;
+      gzochid_application_task *cleanup_task = NULL;
       gzochid_application_task *application_task = NULL;
       gzochid_transactional_application_task_execution *execution = NULL;
       gzochid_task task;
 
-      data[0] = session_oid_str;
-      data[1] = msg;
-      data[2] = &len;
+      data[0] = session_oid;
+      data[1] = g_bytes_new (msg, len);
       
       transactional_task = gzochid_application_task_new
-	(context, gzochid_protocol_client_get_identity (client),
+	(context, gzochid_game_client_get_identity (client),
 	 gzochid_scheme_application_received_message_worker, data);
 
+      cleanup_task = gzochid_application_task_new
+	(context, gzochid_game_client_get_identity (client),
+	 cleanup_received_message_arguments, data);
+      
       execution = gzochid_transactional_application_task_timed_execution_new
-	(transactional_task, NULL, NULL, game_context->tx_timeout);
+	(transactional_task, NULL, cleanup_task, game_context->tx_timeout);
 
-      /* Not necessary to hold a ref to this, as we've transferred them to the
+      /* Not necessary to hold a ref to these, as we've transferred them to the
 	 execution. */
   
       gzochid_application_task_unref (transactional_task);
+      gzochid_application_task_unref (cleanup_task);
       
       application_task = gzochid_application_task_new
-	(context, gzochid_protocol_client_get_identity (client),
-	 gzochid_application_reexecuting_transactional_task_worker, execution);
+	(context, gzochid_game_client_get_identity (client),
+	 gzochid_application_resubmitting_transactional_task_worker, execution);
       
       task.worker = gzochid_application_task_thread_worker;
       task.data = application_task;
       gettimeofday (&task.target_execution_time, NULL);
 
-      gzochid_schedule_run_task (game_context->task_queue, &task);
+      gzochid_schedule_submit_task (game_context->task_queue, &task);
     }
 }
