@@ -1,5 +1,5 @@
 /* scheme-task.c: Scheme callback handlers and interface to gzochi Scheme API
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <assert.h>
 #include <glib.h>
-#include <gmp.h>
 #include <gzochi-common.h>
 #include <libguile.h>
 #include <stddef.h>
@@ -127,8 +127,14 @@ gzochid_scheme_application_initialized_worker
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
 
   callback_reference = gzochid_data_create_reference 
-    (context, &gzochid_scheme_data_serialization, callback);
+    (context, &gzochid_scheme_data_serialization, callback, NULL);
+  
+  /* Creating a reference to a new object requires that the transaction be in a
+     good state, but there's no reason it shouldn't be healthy during 
+     initialization. */
 
+  assert (callback_reference != NULL); 
+  
   gzochid_transaction_join (&scheme_participant, NULL);
 
   gzochid_scheme_invoke_callback 
@@ -161,11 +167,17 @@ scm_to_callback (gzochid_application_context *context, SCM scm_callback)
 {
   GList *module = gzochid_scheme_callback_module (scm_callback);
   char *procedure = gzochid_scheme_callback_procedure (scm_callback);
-  gzochid_data_managed_reference *reference = 
-    gzochid_data_create_reference 
-    (context, &gzochid_scheme_data_serialization, scm_callback);
+  gzochid_scm_location_info *scm_callback_reloc = gzochid_scm_location_get
+    (context, scm_callback);
+  
+  gzochid_data_managed_reference *reference = gzochid_data_create_reference
+    (context, &gzochid_scm_location_aware_serialization, scm_callback_reloc,
+     NULL);
 
-  return gzochid_application_callback_new (procedure, module, reference->oid);
+  if (reference == NULL) /* Can happen on transaction timeout. */
+    return NULL;
+  else return gzochid_application_callback_new
+	 (procedure, module, reference->oid);
 }
 
 static gpointer 
@@ -202,25 +214,20 @@ gzochid_scheme_application_logged_in_worker
  gpointer data)
 {
   GError *err = NULL;
-  mpz_t session_oid;
-  char *session_oid_str = data;
+  guint64 *session_oid = data;
   gzochid_client_session *session = NULL;
   gzochid_data_managed_reference *session_reference = NULL;
 
   SCM scm_session = SCM_BOOL_F;
 
   SCM cb = SCM_BOOL_F;
-  gzochid_data_managed_reference *callback_reference = NULL;
 
   SCM handler = SCM_BOOL_F;
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
     
-  mpz_init (session_oid);
-  mpz_set_str (session_oid, session_oid_str, 16);
   session_reference = gzochid_data_create_reference_to_oid
-    (context, &gzochid_client_session_serialization, session_oid);
-  mpz_clear (session_oid);
-
+    (context, &gzochid_client_session_serialization, *session_oid);
+  
   gzochid_data_dereference (session_reference, &err);
 
   if (err != NULL)
@@ -235,15 +242,13 @@ gzochid_scheme_application_logged_in_worker
     (session, session_reference->oid);
 
   cb = gzochid_scheme_create_callback (context->descriptor->logged_in, NULL);
-  callback_reference = gzochid_data_create_reference 
-    (context, &gzochid_scheme_data_serialization, cb);
-
+  
   gzochid_transaction_join (&scheme_participant, NULL);
 
   handler = gzochid_scheme_invoke_callback 
     (context, identity, "gzochi:execute-logged-in", gzochi_private_app,
-     scm_list_2 (callback_reference->obj, scm_session), exception_var);
-
+     scm_list_2 (cb, scm_session), exception_var);
+  
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
     {
       if (! gzochid_scheme_triggered_by_rollback 
@@ -261,17 +266,30 @@ gzochid_scheme_application_logged_in_worker
     }
   else if (! is_client_session_listener (handler))
     {
-      gzochid_warning ("Invalid type returned by logged-in callback.");
+      g_warning ("Invalid type returned by logged-in callback.");
       gzochid_transaction_mark_for_rollback
 	(&scheme_participant, FALSE);
     }
   else
     {
-      session->handler = gzochid_with_application_context 
-	(context, identity, unpack_handler, handler);
+      gzochid_scm_location_info *scm_handler_reloc = 
+	gzochid_scm_location_get (context, handler);
+      gzochid_data_managed_reference *handler_reference = 
+	gzochid_data_create_reference 
+	(context, &gzochid_scm_location_aware_serialization, scm_handler_reloc,
+	 NULL);
 
+      scm_gc_protect_object (handler);
+      
+      gzochid_client_session_set_handler
+	(session, gzochid_with_application_context 
+	 (context, identity, unpack_handler, handler));
+      gzochid_client_session_set_handler_scm_oid
+	(session, handler_reference->oid);
+      
       gzochid_data_mark 
 	(context, &gzochid_client_session_serialization, session, &err);
+
       if (err == NULL)
 	{
 	  gzochid_scm_location_info *scm_session_reloc = 
@@ -279,12 +297,18 @@ gzochid_scheme_application_logged_in_worker
 	  gzochid_data_managed_reference *reloc_reference = 
 	    gzochid_data_create_reference 
 	    (context, &gzochid_scm_location_aware_serialization, 
-	     scm_session_reloc);
+	     scm_session_reloc, &err);
 
-	  mpz_set (session->scm_oid, reloc_reference->oid);
-	  gzochid_client_session_send_login_success (context, session);
+	  if (err == NULL)
+	    {
+	      gzochid_client_session_set_scm_oid
+		(session, reloc_reference->oid);
+	      gzochid_client_session_send_login_success (context, session);
+	    }
 	}
-      else g_error_free (err);
+      
+      if (err != NULL)
+	g_error_free (err);
     }
 }
 
@@ -301,21 +325,15 @@ gzochid_scheme_application_received_message_worker
   SCM bv = SCM_BOOL_F;
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
 
-  unsigned char *message = NULL;
-  short *message_len_ptr = NULL;
   void **data = ptr;
 
-  mpz_t session_oid;
-  mpz_init (session_oid);
-  mpz_set_str (session_oid, data[0], 16);
-
-  message = data[1];
-  message_len_ptr = data[2];
+  guint64 *session_oid = data[0];
+  size_t message_len;
+  unsigned char *message = (unsigned char *) g_bytes_get_data
+    (data[1], &message_len);
 
   session_reference = gzochid_data_create_reference_to_oid 
-    (context, &gzochid_client_session_serialization, session_oid);
-
-  mpz_clear (session_oid);
+    (context, &gzochid_client_session_serialization, *session_oid);
 
   gzochid_data_dereference (session_reference, &err);
 
@@ -327,11 +345,12 @@ gzochid_scheme_application_received_message_worker
 
   session = session_reference->obj;
 
-  bv = gzochid_scheme_create_bytevector (message, *message_len_ptr);
+  bv = gzochid_scheme_create_bytevector (message, message_len);
+  
   callback_reference =
     gzochid_data_create_reference_to_oid
-    (context, &gzochid_scheme_data_serialization, 
-     session->handler->received_message->scm_oid);
+    (context, &gzochid_scheme_data_serialization,
+     gzochid_client_session_get_handler (session)->received_message->scm_oid);
 
   gzochid_data_dereference (callback_reference, &err);
 
@@ -368,66 +387,69 @@ gzochid_scheme_application_disconnected_worker
   gzochid_client_session *session = NULL;
   gzochid_data_managed_reference *session_reference = NULL;
   gzochid_data_managed_reference *callback_reference = NULL;
-
+  gzochid_client_session_handler *handler = NULL;
+  
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
-  char *oid_str = ptr;
-  mpz_t session_oid;
-
-  mpz_init (session_oid);
-  mpz_set_str (session_oid, oid_str, 16);
+  guint64 *session_oid = ptr;
 
   session_reference = gzochid_data_create_reference_to_oid 
-    (context, &gzochid_client_session_serialization, session_oid);
-  mpz_clear (session_oid);
+    (context, &gzochid_client_session_serialization, *session_oid);
 
-  gzochid_data_dereference (session_reference, &err);
+  /* Obtain a preemptive write lock on the session object, as it's most likely 
+     going to be removed later in the same transaction. */
+  
+  gzochid_data_dereference_for_update (session_reference, &err);
 
   if (err != NULL)
     {
       if (err->message != NULL)
-	gzochid_warning
-	  ("Failed to dereference disconnecting session '%s': %s", 
-	   oid_str, err->message);
-      else gzochid_warning 
-	     ("Failed to dereference disconnecting session '%s'.", oid_str);
+	g_warning
+	  ("Failed to dereference disconnecting session '%lx': %s", 
+	   *session_oid, err->message);
+      else g_warning 
+	     ("Failed to dereference disconnecting session '%lx'.",
+	      *session_oid);
 
       g_error_free (err);
       return;
     }
 
   session = session_reference->obj;
-
+  handler = gzochid_client_session_get_handler (session);
+  
   /* A client may disconnect before the login process has completed or after
      it has failed enough times to stop being retried. */
 
-  if (session->handler == NULL)
+  if (handler == NULL)
     {
       /* In that case, don't bother trying to dredge up its disconnect handler.
 	 Just jump straight to removing the session from the data store. */
       
       gzochid_tx_warning 
-	(context, "Session '%s' disconnected after incomplete login.", ptr);
+	(context, "Session '%lx' disconnected after incomplete login.",
+	 *session_oid);
       gzochid_client_session_disconnected_worker (context, identity, ptr);
       
       return;
     }
 
-  callback_reference =
-    gzochid_data_create_reference_to_oid
-    (context, &gzochid_scheme_data_serialization, 
-     session->handler->disconnected->scm_oid);
+  callback_reference = gzochid_data_create_reference_to_oid
+    (context, &gzochid_scheme_data_serialization,
+     handler->disconnected->scm_oid);
 
-  gzochid_data_dereference (callback_reference, &err);
+  /* Mark the callback for update; if all goes well, it'll get removed below. */
+  
+  gzochid_data_dereference_for_update (callback_reference, &err);
   
   if (err != NULL)
     {
       if (err->message != NULL)
-	gzochid_warning
-	  ("Failed to retrieve disconnect callback for session '%s': %s", 
-	   oid_str, err->message);
-      else gzochid_warning 
-	     ("Failed to retrieve disconnect callback for session '%s'.", 
-	      oid_str);
+	g_warning
+	  ("Failed to retrieve disconnect callback for session '%lx': %s", 
+	   *session_oid, err->message);
+      else g_warning 
+	     ("Failed to retrieve disconnect callback for session '%lx'.", 
+	      *session_oid);
 
       g_error_free (err);      
       return;
@@ -449,7 +471,29 @@ gzochid_scheme_application_disconnected_worker
 	     (scm_variable_ref (exception_var)));
 	}
     }
-  else gzochid_client_session_disconnected_worker (context, identity, ptr);
+  else
+    {
+      gzochid_data_remove_object (callback_reference, &err);
+
+      if (err == NULL)
+	{
+	  callback_reference = gzochid_data_create_reference_to_oid
+	    (context, &gzochid_scheme_data_serialization, 
+	     handler->received_message->scm_oid);
+
+	  gzochid_data_remove_object (callback_reference, &err);
+	}
+      
+      if (err != NULL)
+	{
+	  g_warning
+	    ("Failed to remove lifecycle handler for session '%lx': %s",
+	     *session_oid,  err->message);
+	  
+	  g_error_free (err);
+	}
+      else gzochid_client_session_disconnected_worker (context, identity, ptr);
+    }
 }
 
 void 
@@ -491,12 +535,13 @@ gzochid_scheme_application_ready (gzochid_application_context *context,
 
 static void 
 scheme_worker_serializer (gzochid_application_context *context, 
-			  gzochid_application_worker worker, GString *out)
+			  gzochid_application_worker worker, GByteArray *out)
 {
 }
 
 static gzochid_application_worker 
-scheme_worker_deserializer (gzochid_application_context *context, GString *in)
+scheme_worker_deserializer (gzochid_application_context *context,
+			    GByteArray *in)
 {
   return gzochid_scheme_application_task_worker;
 }
