@@ -1,5 +1,5 @@
 /* session.c: Client session management routines for gzochid
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -27,10 +27,9 @@
 #include "data.h"
 #include "event.h"
 #include "game.h"
+#include "game-protocol.h"
 #include "gzochid-auth.h"
 #include "io.h"
-#include "log.h"
-#include "protocol.h"
 #include "reloc.h"
 #include "scheme-task.h"
 #include "session.h"
@@ -49,10 +48,29 @@ enum gzochid_client_session_operation
     GZOCHID_CLIENT_SESSION_OP_MESSAGE
   };
 
+struct _gzochid_client_session
+{
+  gzochid_auth_identity *identity;
+  gzochid_client_session_handler *handler;
+  gboolean connected;
+
+  guint64 scm_oid; /* Object id of the Scheme representation of the session. */
+
+  /* Object id of the Scheme representation of the client session listener. This
+     field is tracked purely to keep track of session listener, which is a
+     managed record and thus winds up in the database. Although from the point
+     of you of the container, the only thing the listener is used for is its two
+     lifecycle callbacks, it's not easy to predict what else application code 
+     will do with it. For that reason, it has to stick around at least as long
+     as the session does. */
+ 
+  guint64 handler_scm_oid;
+};
+
 struct _gzochid_client_session_pending_operation
 {
   enum gzochid_client_session_operation type;
-  mpz_t target_session;
+  guint64 target_session;
 };
 
 typedef struct _gzochid_client_session_pending_operation
@@ -100,46 +118,43 @@ create_transaction_context (gzochid_application_context *context)
 }
 
 static gzochid_client_session_pending_operation *
-create_disconnect_operation (mpz_t target_session)
+create_disconnect_operation (guint64 target_session)
 {
   gzochid_client_session_pending_operation *op = 
     malloc (sizeof (gzochid_client_session_pending_operation));
 
   op->type = GZOCHID_CLIENT_SESSION_OP_DISCONNECT;
-  mpz_init (op->target_session);
-  mpz_set (op->target_session, target_session);
+  op->target_session = target_session;
 
   return op;
 }
 
 static gzochid_client_session_pending_operation *
-create_login_success_operation (mpz_t target_session)
+create_login_success_operation (guint64 target_session)
 {
   gzochid_client_session_pending_operation *op = 
     malloc (sizeof (gzochid_client_session_pending_operation));
 
   op->type = GZOCHID_CLIENT_SESSION_OP_LOGIN_SUCCESS;
-  mpz_init (op->target_session);
-  mpz_set (op->target_session, target_session);
+  op->target_session = target_session;
 
   return op;  
 }
 
 static gzochid_client_session_pending_operation *
-create_login_failure_operation (mpz_t target_session)
+create_login_failure_operation (guint64 target_session)
 {
   gzochid_client_session_pending_operation *op = 
     malloc (sizeof (gzochid_client_session_pending_operation));
 
   op->type = GZOCHID_CLIENT_SESSION_OP_LOGIN_FAILURE;
-  mpz_init (op->target_session);
-  mpz_set (op->target_session, target_session);
+  op->target_session = target_session;
 
   return op;  
 }
 
 static gzochid_client_session_pending_operation *
-create_message_operation (mpz_t target_session, unsigned char *msg, short len)
+create_message_operation (guint64 target_session, unsigned char *msg, short len)
 {
   gzochid_client_session_pending_message_operation *msg_op = 
     malloc (sizeof (gzochid_client_session_pending_message_operation));
@@ -147,8 +162,7 @@ create_message_operation (mpz_t target_session, unsigned char *msg, short len)
     (gzochid_client_session_pending_operation *) msg_op;
 
   op->type = GZOCHID_CLIENT_SESSION_OP_MESSAGE;
-  mpz_init (op->target_session);
-  mpz_set (op->target_session, target_session);
+  op->target_session = target_session;
 
   msg_op->message = msg;
   msg_op->len = len;
@@ -162,22 +176,13 @@ session_prepare (gpointer data)
   return TRUE;
 }
 
-static void 
-free_operation (gpointer data)
-{
-  gzochid_client_session_pending_operation *op = data;
-
-  mpz_clear (op->target_session);
-  free (op);
-}
-
 static void 
 cleanup_transaction (gzochid_client_session_transaction_context *tx_context)
 {
-  g_list_free_full (tx_context->operations, free_operation);
+  g_list_free_full (tx_context->operations, free);
 
   if (tx_context->login_operation != NULL)
-    free_operation (tx_context->login_operation);
+    free (tx_context->login_operation);
   
   free (tx_context);
 }
@@ -189,43 +194,39 @@ session_commit_operation
 {
   gzochid_application_context *context = tx_context->context;
   gzochid_client_session_pending_message_operation *msg_op = NULL;
-
-  char *oid_str = mpz_get_str (NULL, 16, op->target_session);
-  gzochid_protocol_client *client = NULL;
+  gzochid_game_client *client = NULL;
 
   g_mutex_lock (&context->client_mapping_lock);
-  client = g_hash_table_lookup (context->oids_to_clients, oid_str);
+  client = g_hash_table_lookup (context->oids_to_clients, &op->target_session);
 
   if (client == NULL)
     {
-      gzochid_warning 
-	("Client not found for session '%s'; skipping operation.", oid_str);
+      g_warning
+	("Client not found for session '%lx'; skipping operation.",
+	 op->target_session);
+
       g_mutex_unlock (&context->client_mapping_lock);
-      free (oid_str);
       return;
     }
 
-  free (oid_str);
-
   switch (op->type)
     {
     case GZOCHID_CLIENT_SESSION_OP_DISCONNECT:
-      gzochid_protocol_client_disconnect (client); break;
+      gzochid_game_client_disconnect (client); break;
     case GZOCHID_CLIENT_SESSION_OP_LOGIN_SUCCESS:
-      gzochid_protocol_client_login_success (client); break;
+      gzochid_game_client_login_success (client); break;
     case GZOCHID_CLIENT_SESSION_OP_LOGIN_FAILURE:
-      gzochid_protocol_client_login_failure (client); break;
+      gzochid_game_client_login_failure (client); break;
     case GZOCHID_CLIENT_SESSION_OP_MESSAGE:
 
       if (!tx_context->login_failed 
-	  || mpz_cmp (op->target_session, 
-		      tx_context->login_operation->target_session) != 0)
+	  || op->target_session != tx_context->login_operation->target_session) 
 	{
 	  msg_op = (gzochid_client_session_pending_message_operation *) op;
 	  gzochid_application_event_dispatch
 	    (context->event_source,
 	     gzochid_application_event_new (MESSAGE_SENT));
-	  gzochid_protocol_client_send (client, msg_op->message, msg_op->len);
+	  gzochid_game_client_send (client, msg_op->message, msg_op->len);
 	}
 
       break;
@@ -269,7 +270,7 @@ session_rollback (gpointer data)
 }
 
 static gzochid_client_session_handler *
-deserialize_handler (gzochid_application_context *context, GString *in, 
+deserialize_handler (gzochid_application_context *context, GByteArray *in, 
 		     GError **err)
 {
   gzochid_client_session_handler *handler = malloc 
@@ -284,23 +285,21 @@ deserialize_handler (gzochid_application_context *context, GString *in,
 }
 
 static gpointer 
-deserialize_client_session (gzochid_application_context *context, GString *in, 
-			    GError **err)
+deserialize_client_session (gzochid_application_context *context,
+			    GByteArray *in, GError **err)
 {
   gzochid_auth_identity *identity = 
     gzochid_auth_identity_deserializer (context, in, NULL);
-  GSequence *channels = gzochid_util_deserialize_sequence 
-    (in, (gpointer (*) (GString *)) gzochid_util_deserialize_string, free);
   gzochid_client_session *session = gzochid_client_session_new (identity);
 
-  g_sequence_free (session->channels);
-  session->channels = channels;
-
-  gzochid_util_deserialize_mpz (in, session->scm_oid);
+  session->scm_oid = gzochid_util_deserialize_oid (in);
 
   if (gzochid_util_deserialize_boolean (in))
-    session->handler = deserialize_handler (context, in, NULL);
-
+    {
+      session->handler = deserialize_handler (context, in, NULL);
+      session->handler_scm_oid = gzochid_util_deserialize_oid (in);
+    }
+  
   gzochid_auth_identity_unref (identity);
   
   return session;
@@ -308,7 +307,7 @@ deserialize_client_session (gzochid_application_context *context, GString *in,
 
 static void 
 serialize_handler (gzochid_application_context *context, 
-		   gzochid_client_session_handler *handler, GString *out, 
+		   gzochid_client_session_handler *handler, GByteArray *out, 
 		   GError **err)
 {
   gzochid_application_callback_serialization.serializer 
@@ -319,19 +318,18 @@ serialize_handler (gzochid_application_context *context,
 
 static void 
 serialize_client_session (gzochid_application_context *context, gpointer obj, 
-			  GString *out, GError **err)
+			  GByteArray *out, GError **err)
 {
   gzochid_client_session *session = obj;
 
   gzochid_auth_identity_serializer (context, session->identity, out, NULL);
-  gzochid_util_serialize_sequence 
-    (session->channels, 
-     (void (*) (gpointer, GString *)) gzochid_util_serialize_string, out);
-  gzochid_util_serialize_mpz (session->scm_oid, out);
+  gzochid_util_serialize_oid (session->scm_oid, out);
+
   if (session->handler != NULL)
     {
       gzochid_util_serialize_boolean (TRUE, out);
       serialize_handler (context, session->handler, out, NULL);
+      gzochid_util_serialize_oid (session->handler_scm_oid, out);
     }
   else gzochid_util_serialize_boolean (FALSE, out);
 }
@@ -353,8 +351,6 @@ finalize_client_session (gzochid_application_context *context, gpointer obj)
   gzochid_client_session *session = obj;
 
   gzochid_auth_identity_finalizer (context, session->identity);
-  g_sequence_free (session->channels);
-  mpz_clear (session->scm_oid);
 
   if (session->handler != NULL)
     finalize_handler (context, session->handler);
@@ -373,24 +369,20 @@ gzochid_io_serialization gzochid_client_session_serialization =
   };
 
 static void 
-remove_session (gzochid_application_context *context, const char *oid_str, 
+remove_session (gzochid_application_context *context, guint64 session_oid, 
 		GError **err)
 {
   GError *local_err = NULL;
   GString *binding = g_string_new (SESSION_PREFIX);
 
-  g_string_append (binding, oid_str);
+  g_string_printf (binding, "%lx", session_oid);
   gzochid_data_remove_binding (context, binding->str, &local_err);
   if (local_err == NULL)
     {
-      mpz_t oid;
       gzochid_data_managed_reference *session_reference = NULL;
 
-      mpz_init (oid);
-      mpz_set_str (oid, oid_str, 16);
-
       session_reference = gzochid_data_create_reference_to_oid 
-	(context, &gzochid_client_session_serialization, oid);
+	(context, &gzochid_client_session_serialization, session_oid);
       gzochid_data_dereference (session_reference, &local_err);
 
       if (local_err == NULL)
@@ -400,6 +392,10 @@ remove_session (gzochid_application_context *context, const char *oid_str,
 	    gzochid_data_create_reference_to_oid 
 	    (context, &gzochid_scm_location_aware_serialization, 
 	     session->scm_oid);
+	  gzochid_data_managed_reference *scm_handler_reference =
+	    gzochid_data_create_reference_to_oid 
+	    (context, &gzochid_scm_location_aware_serialization, 
+	     session->handler_scm_oid);
 	  
 	  gzochid_data_remove_object (scm_session_reference, &local_err);
 
@@ -411,26 +407,38 @@ remove_session (gzochid_application_context *context, const char *oid_str,
 		 successfully. */
 
 	      if (local_err->message != NULL)
-		gzochid_info 
-		  ("Unable to remove Scheme object for session '%s': %s",
-		   oid_str, local_err->message);
+		g_info
+		  ("Unable to remove Scheme object for session '%lx': %s",
+		   session_oid, local_err->message);
 	    }
 
-	  /* Don't attempt to remove the object if the transaction is in a
-	     failed state - it's a waste of time and tends to irritate the 
-	     storage engine. */
+	  if (local_err == NULL ||
+	      !g_error_matches (local_err, GZOCHID_DATA_ERROR,
+				GZOCHID_DATA_ERROR_TRANSACTION))
+	    g_clear_error (&local_err);
+
+	  /* Now do the same thing for the Scheme representation of the
+	     client lifecycle handler. */
+
+	  if (local_err == NULL)
+	    gzochid_data_remove_object (scm_handler_reference, &local_err);
+
+	  if (local_err == NULL ||
+	      !g_error_matches (local_err, GZOCHID_DATA_ERROR,
+				GZOCHID_DATA_ERROR_TRANSACTION))
+	    g_clear_error (&local_err);
 	  
-	  if (!g_error_matches
-	      (local_err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION))
-	    {
-	      g_clear_error (&local_err);
-	      gzochid_data_remove_object (session_reference, &local_err);
-	    }
+	  if (local_err == NULL)
+
+	    /* Don't attempt to remove the object if the transaction is in a
+	       failed state - it's a waste of time and tends to irritate the 
+	       storage engine. */
+	    
+	    gzochid_data_remove_object (session_reference, &local_err);
 	}
+      
       if (local_err != NULL)
-	g_propagate_error (err, local_err);	  
-	
-      mpz_clear (oid);
+	g_propagate_error (err, local_err);	  	
     }
   else g_propagate_error (err, local_err);
    
@@ -442,7 +450,16 @@ gzochid_client_session_disconnected_worker
 (gzochid_application_context *context, gzochid_auth_identity *identity,
  gpointer data)
 {
-  remove_session (context, data, NULL);
+  GError *err = NULL;
+  guint64 *oid = data;
+  
+  remove_session (context, *oid, &err);
+
+  if (err != NULL)
+    {
+      g_debug ("Failed to remove session: %s", err->message);
+      g_error_free (err);
+    }
 }
 
 gzochid_client_session *
@@ -452,9 +469,7 @@ gzochid_client_session_new (gzochid_auth_identity *identity)
     calloc (1, sizeof (gzochid_client_session));
 
   session->identity = gzochid_auth_identity_ref (identity);
-  session->channels = g_sequence_new (NULL);
-  mpz_init (session->scm_oid);
-
+  
   return session;
 }
 
@@ -462,8 +477,6 @@ void
 gzochid_client_session_free (gzochid_client_session *session)
 {
   gzochid_auth_identity_unref (session->identity);
-  g_sequence_free (session->channels);
-  mpz_clear (session->scm_oid);
   
   free (session);
 }
@@ -489,24 +502,67 @@ gzochid_client_session_disconnect (gzochid_application_context *context,
 				   gzochid_client_session *session)
 {
   GError *err = NULL;
-  char *oid_str = NULL;
   gzochid_client_session_transaction_context *tx_context = 
     join_transaction (context);
   gzochid_data_managed_reference *reference = gzochid_data_create_reference 
-    (context, &gzochid_client_session_serialization, session);
+    (context, &gzochid_client_session_serialization, session, NULL);
 
+  /* A reference to the session should already exist in the transaction. */
+  
+  assert (reference != NULL);
   assert (reference->state != GZOCHID_MANAGED_REFERENCE_STATE_NEW);
 
-  oid_str = mpz_get_str (NULL, 16, reference->oid);
-
-  remove_session (context, oid_str, &err);
+  remove_session (context, reference->oid, &err);
 
   if (err == NULL)
     tx_context->operations = g_list_append 
       (tx_context->operations, create_disconnect_operation (reference->oid));
   else g_error_free (err);
+}
 
-  free (oid_str);
+gzochid_auth_identity *
+gzochid_client_session_identity (gzochid_client_session *session)
+{
+  return session->identity;
+}
+
+guint64
+gzochid_client_session_scm_oid (gzochid_client_session *session)
+{
+  return session->scm_oid;
+}
+
+void
+gzochid_client_session_set_scm_oid (gzochid_client_session *session,
+				    guint64 scm_oid)
+{
+  session->scm_oid = scm_oid;
+}
+
+guint64
+gzochid_client_session_handler_scm_oid (gzochid_client_session *session)
+{
+  return session->handler_scm_oid;
+}
+
+void
+gzochid_client_session_set_handler_scm_oid (gzochid_client_session *session,
+					    guint64 handler_scm_oid)
+{
+  session->handler_scm_oid = handler_scm_oid;
+}
+
+gzochid_client_session_handler *
+gzochid_client_session_get_handler (gzochid_client_session *session)
+{
+  return session->handler;
+}
+
+void
+gzochid_client_session_set_handler (gzochid_client_session *session,
+				    gzochid_client_session_handler *handler)
+{
+  session->handler = handler;
 }
 
 void 
@@ -516,8 +572,11 @@ gzochid_client_session_send_login_success (gzochid_application_context *context,
   gzochid_client_session_transaction_context *tx_context =
     join_transaction (context);
   gzochid_data_managed_reference *reference = gzochid_data_create_reference 
-    (context, &gzochid_client_session_serialization, session);
+    (context, &gzochid_client_session_serialization, session, NULL);
 
+  /* A reference to the session should already exist in the transaction. */
+
+  assert (reference != NULL);
   assert (reference->state != GZOCHID_MANAGED_REFERENCE_STATE_NEW);
   assert (tx_context->login_operation == NULL);
   
@@ -532,8 +591,11 @@ gzochid_client_session_send_login_failure (gzochid_application_context *context,
   gzochid_client_session_transaction_context *tx_context =
     join_transaction (context);
   gzochid_data_managed_reference *reference = gzochid_data_create_reference 
-    (context, &gzochid_client_session_serialization, session);
+    (context, &gzochid_client_session_serialization, session, NULL);
 
+  /* A reference to the session should already exist in the transaction. */
+
+  assert (reference != NULL);
   assert (reference->state != GZOCHID_MANAGED_REFERENCE_STATE_NEW);
   assert (tx_context->login_operation == NULL);
 
@@ -549,8 +611,11 @@ gzochid_client_session_send_message (gzochid_application_context *context,
   gzochid_client_session_transaction_context *tx_context =
     join_transaction (context);
   gzochid_data_managed_reference *reference = gzochid_data_create_reference 
-    (context, &gzochid_client_session_serialization, session);
+    (context, &gzochid_client_session_serialization, session, NULL);
 
+  /* A reference to the session should already exist in the transaction. */
+
+  assert (reference != NULL);
   assert (reference->state != GZOCHID_MANAGED_REFERENCE_STATE_NEW);
 
   tx_context->operations = g_list_append 
@@ -563,101 +628,80 @@ struct _gzochid_persistence_task_data
   gpointer data;
   gzochid_io_serialization *serialization;
   gzochid_oid_holder *holder;
+  char *prefix;
 };
 
 typedef struct _gzochid_persistence_task_data gzochid_persistence_task_data;
 
-struct _gzochid_prefix_binding_persistence_task_data
+static gzochid_persistence_task_data *
+gzochid_persistence_task_data_new (gzochid_io_serialization *serialization,
+				   gpointer data, gzochid_oid_holder *holder,
+				   char *prefix)
 {
-  gzochid_persistence_task_data base;
+  gzochid_persistence_task_data *task =
+    malloc (sizeof (gzochid_persistence_task_data));
 
-  char *prefix;
-};
-
-typedef struct _gzochid_prefix_binding_persistence_task_data
- gzochid_prefix_binding_persistence_task_data;
-
-static void 
-init_persistence_task (gzochid_persistence_task_data *task, 
-		       gzochid_io_serialization *serialization, gpointer data, 
-		       gzochid_oid_holder *holder)
-{
   task->data = data;
   task->serialization = serialization;
   task->holder = holder;
-} 
-
-static gzochid_prefix_binding_persistence_task_data *
-gzochid_prefix_binding_persistence_task_data_new
-(gzochid_io_serialization *serialization, gpointer data,
- gzochid_oid_holder *holder, char *prefix)
-{
-  gzochid_prefix_binding_persistence_task_data *task = 
-    malloc (sizeof (gzochid_prefix_binding_persistence_task_data));
-
-  init_persistence_task 
-    ((gzochid_persistence_task_data *) task, serialization, data, holder);
   task->prefix = prefix;
 
   return task;
 }
 
 static void 
-persistence_task_worker (gzochid_application_context *context, 
+persistence_task_worker (gzochid_application_context *context,
 			 gzochid_auth_identity *identity, gpointer data)
 {
+  GError *local_err = NULL;
   gzochid_persistence_task_data *persistence_task = data;
-  gzochid_data_managed_reference *reference = 
-    gzochid_data_create_reference 
-    (context, persistence_task->serialization, persistence_task->data);
+  GString *binding = NULL;
+
+  gzochid_data_managed_reference *reference = gzochid_data_create_reference 
+    (context, persistence_task->serialization, persistence_task->data,
+     &local_err);
+
+  if (local_err != NULL)
+    {
+      g_propagate_error (&persistence_task->holder->err, local_err);
+      return;
+    }
+
+  persistence_task->holder->oid = reference->oid;
+
+  binding = g_string_new (persistence_task->prefix);
+  g_string_printf (binding, "%lx", reference->oid);
+
+  g_clear_error (&persistence_task->holder->err);
   
-  mpz_set (persistence_task->holder->oid, reference->oid);
-}
-
-static void 
-prefix_task_worker (gzochid_application_context *context,
-		    gzochid_auth_identity *identity, gpointer data)
-{
-  gzochid_prefix_binding_persistence_task_data *
-    prefix_binding_persistence_task = data;
-  gzochid_persistence_task_data *persistence_task =
-    (gzochid_persistence_task_data *) prefix_binding_persistence_task;
-  GString *binding = g_string_new (prefix_binding_persistence_task->prefix);
-  char *oid_str = NULL;
-
-  persistence_task_worker (context, identity, data);
-  
-  oid_str = mpz_get_str (NULL, 16, persistence_task->holder->oid);
-  g_string_append (binding, oid_str);
-  free (oid_str);
-
   gzochid_data_set_binding_to_oid 
-    (context, binding->str, persistence_task->holder->oid, NULL);
+    (context, binding->str, persistence_task->holder->oid, &local_err);
 
+  if (local_err != NULL)
+    g_propagate_error (&persistence_task->holder->err, local_err);
+  
   g_string_free (binding, TRUE);
 }
 
 static void
-prefix_task_cleanup_worker (gzochid_application_context *context,
-			    gzochid_auth_identity *identity, gpointer data)
+persistence_task_cleanup_worker (gzochid_application_context *context,
+				 gzochid_auth_identity *identity, gpointer data)
 {
   free (data);
 }
 
 gzochid_task *
-gzochid_data_prefix_binding_persistence_task_new
+gzochid_data_persistence_task_new
 (gzochid_application_context *context, gzochid_auth_identity *identity, 
  gzochid_io_serialization *serialization, gpointer data, 
  gzochid_oid_holder *holder, char *prefix)
 {
-  gzochid_prefix_binding_persistence_task_data *task_data =
-    gzochid_prefix_binding_persistence_task_data_new 
+  gzochid_persistence_task_data *task_data = gzochid_persistence_task_data_new 
     (serialization, data, holder, prefix);
-
   gzochid_application_task *task = gzochid_application_task_new
-    (context, identity, prefix_task_worker, task_data);
+    (context, identity, persistence_task_worker, task_data);
   gzochid_application_task *cleanup_task = gzochid_application_task_new 
-    (context, identity, prefix_task_cleanup_worker, task_data);
+    (context, identity, persistence_task_cleanup_worker, task_data);
   gzochid_transactional_application_task_execution *execution = 
     gzochid_transactional_application_task_execution_new
     (task, NULL, cleanup_task);
@@ -683,19 +727,26 @@ gzochid_data_prefix_binding_persistence_task_new
 
 void 
 gzochid_client_session_persist (gzochid_application_context *context, 
-				gzochid_client_session *session, mpz_t oid)
+				gzochid_client_session *session, guint64 *oid,
+				GError **err)
 {
   gzochid_game_context *game_context = 
     (gzochid_game_context *) ((gzochid_context *) context)->parent;
   gzochid_oid_holder *holder = gzochid_oid_holder_new ();
-  gzochid_task *task = gzochid_data_prefix_binding_persistence_task_new
-    (context, session->identity, &gzochid_client_session_serialization, 
-     session, holder, SESSION_PREFIX);
+  gzochid_task *task = gzochid_data_persistence_task_new
+    (context, session->identity, &gzochid_client_session_serialization, session,
+     holder, SESSION_PREFIX);
 
   gzochid_schedule_run_task (game_context->task_queue, task);  
   gzochid_task_free (task);
 
-  mpz_set (oid, holder->oid);
+  if (holder->err != NULL)
+    {
+      g_propagate_error (err, holder->err);
+      holder->err = NULL;
+    }
+  else *oid = holder->oid;
+  
   gzochid_oid_holder_free (holder);
 }
 
@@ -703,44 +754,38 @@ void
 gzochid_sweep_client_sessions (gzochid_application_context *context,
 			       GError **err)
 {
-  mpz_t oid;
   GError *local_err = NULL;
   char *next_binding = NULL;
   int prefix_len = strlen (SESSION_PREFIX), num_sessions = 0;
+  guint64 oid = 0;
 
-  mpz_init (oid);
   gzochid_tx_info (context, "Sweeping old client sessions.");
   next_binding = gzochid_data_next_binding_oid 
-    (context, SESSION_PREFIX, oid, &local_err);
+    (context, SESSION_PREFIX, &oid, &local_err);
 
   assert (local_err == NULL);
 
   while (next_binding != NULL 
          && strncmp (SESSION_PREFIX, next_binding, prefix_len) == 0)
     {
-      char *oid_str = mpz_get_str (NULL, 16, oid);
       char *next_next_binding = NULL;
       
       gzochid_scheme_application_disconnected_worker 
-	(context, gzochid_auth_system_identity (), oid_str);      
+	(context, gzochid_auth_system_identity (), &oid);
 
       if (gzochid_transaction_rollback_only ())
 	{
 	  g_set_error
 	    (err, GZOCHID_SESSION_ERROR, GZOCHID_SESSION_ERROR_DISCONNECT,
-	     "Transaction rollback while sweeping disconnected session '%s'.",
-	     oid_str);
+	     "Transaction rollback while sweeping disconnected session '%lx'.",
+	     oid);
 	  
 	  free (next_binding);
-	  free (oid_str);
-	  mpz_clear (oid);
-	  
 	  return;
 	}
 
-      free (oid_str);
       next_next_binding = gzochid_data_next_binding_oid 
-	(context, next_binding, oid, &local_err);
+	(context, next_binding, &oid, &local_err);
 
       assert (local_err == NULL);
 
@@ -750,5 +795,4 @@ gzochid_sweep_client_sessions (gzochid_application_context *context,
     }
 
   gzochid_tx_info (context, "Swept %d session(s).", num_sessions);
-  mpz_clear (oid);  
 }
