@@ -1,5 +1,5 @@
 /* tx.c: Application-level transactions implementation for gzochid
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -17,20 +17,17 @@
 
 #include <assert.h>
 #include <glib.h>
-#include <gmp.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <sys/time.h>
 
-#include "log.h"
 #include "tx.h"
 #include "util.h"
 
 static GMutex transaction_mutex;
 static GPrivate thread_transaction_key;
 static GPrivate thread_transaction_timing_key;
-static mpz_t transaction_counter;
-static gboolean transactions_initialized;
+static guint64 transaction_counter;
 
 enum gzochid_transaction_state
   {
@@ -44,7 +41,7 @@ enum gzochid_transaction_state
 
 struct _gzochid_transaction 
 {
-  mpz_t id;
+  guint64 id;
   char *name;
   GHashTable *participants;
   enum gzochid_transaction_state state;
@@ -54,24 +51,18 @@ struct _gzochid_transaction
 
 typedef struct _gzochid_transaction gzochid_transaction;
 
-static 
-gzochid_transaction *transaction_new (void)
+static gzochid_transaction *
+transaction_new (void)
 {
   gzochid_transaction *transaction = calloc (1, sizeof (gzochid_transaction));
   transaction->participants = g_hash_table_new_full 
     (g_str_hash, g_str_equal, NULL, free);
 
   g_mutex_lock (&transaction_mutex);
-  if (!transactions_initialized)
-    {
-      mpz_init (transaction_counter);
-      transactions_initialized = TRUE;
-    }
-  mpz_set (transaction->id, transaction_counter);
-  mpz_add_ui (transaction_counter, transaction_counter, 1);
+  transaction->id = transaction_counter++;
   g_mutex_unlock (&transaction_mutex);
 
-  transaction->name = mpz_get_str (NULL, 16, transaction->id);
+  transaction->name = g_strdup_printf ("%lx", transaction->id);
   transaction->state = GZOCHID_TRANSACTION_STATE_ACTIVE;
 
   return transaction;
@@ -80,8 +71,7 @@ gzochid_transaction *transaction_new (void)
 static void 
 transaction_free (gzochid_transaction *transaction)
 {
-  mpz_clear (transaction->id);
-  free (transaction->name);
+  g_free (transaction->name);
   g_hash_table_destroy (transaction->participants);
   free (transaction);
 }
@@ -147,8 +137,8 @@ gzochid_transaction_join (gzochid_transaction_participant *participant,
       g_hash_table_insert 
 	(transaction->participants, participant->name, registration);
 
-      gzochid_debug ("Participant '%s' joined transaction '%s'.", 
-		     participant->name, transaction->name);
+      g_debug ("Participant '%s' joined transaction '%s'.", participant->name,
+	       transaction->name);
     }
 }
 
@@ -173,7 +163,7 @@ prepare (gzochid_transaction *transaction)
   GList *participant_ptr = participants;
 
   transaction->state = GZOCHID_TRANSACTION_STATE_PREPARING;
-  gzochid_debug ("Preparing transaction '%s' for commit.", transaction->name);
+  g_debug ("Preparing transaction '%s' for commit.", transaction->name);
   
   while (participant_ptr != NULL)
     {
@@ -181,7 +171,7 @@ prepare (gzochid_transaction *transaction)
 	(gzochid_transaction_participant_registration *) participant_ptr->data;
       if (!participant->participant->prepare (participant->data))
 	{
-	  gzochid_info 
+	  g_info 
 	    ("Participant '%s' in transaction '%s' failed to prepare.",
 	     participant->participant->name, transaction->name);
 
@@ -213,7 +203,7 @@ commit (gzochid_transaction *transaction)
     }
 
   transaction->state = GZOCHID_TRANSACTION_STATE_COMMITTED;
-  gzochid_debug ("Committed transaction '%s'.", transaction->name);
+  g_debug ("Committed transaction '%s'.", transaction->name);
 
   g_list_free (participants);
 }
@@ -271,7 +261,7 @@ rollback (gzochid_transaction *transaction)
     }
 
   transaction->state = GZOCHID_TRANSACTION_STATE_ROLLED_BACK;
-  gzochid_debug ("Rolled back transaction '%s'.", transaction->name);
+  g_debug ("Rolled back transaction '%s'.", transaction->name);
 
   g_list_free (participants);
 }
@@ -379,8 +369,8 @@ gzochid_transaction_end ()
 	+ ((tx_finish.tv_usec - timing->start_time.tv_usec) / 1000);
       
       if (success)
-	gzochid_debug ("Transaction completed in %dms", ms);
-      else gzochid_warning ("Transaction failed after %dms", ms);
+	g_debug ("Transaction completed in %dms", ms);
+      else g_warning ("Transaction failed after %dms", ms);
     }
 
   g_private_set (&thread_transaction_key, NULL);
