@@ -1,5 +1,5 @@
 /* api/test-tx.c: Test routines for api/tx.c in gzochid.
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -21,10 +21,36 @@
 #include <stdlib.h>
 
 #include "../app.h"
+#include "../channel.h"
+#include "../guile.h"
 #include "../gzochid-auth.h"
+#include "../session.h"
 #include "../tx.h"
 
 #include "tx.h"
+#include "util.h"
+
+/* Fake implementation to avoid having to pull in `channel.o'. */
+
+const char *
+gzochid_channel_name (gzochid_channel *channel)
+{
+  return NULL;
+}
+
+/* Fake implementation to avoid having to pull in `session.o'. */
+
+gzochid_auth_identity *
+gzochid_client_session_identity (gzochid_client_session *session)
+{
+  return NULL;
+}
+
+extern void
+primitive_join_transaction (SCM);
+
+extern void
+primitive_abort_transaction (SCM, SCM);
 
 struct test_context
 {
@@ -111,7 +137,7 @@ SCM
 abort_transaction_body (void *data)
 {
   SCM participant = data;
-  primitive_abort_transaction (participant);
+  primitive_abort_transaction (participant, SCM_BOOL_F);
   return SCM_UNSPECIFIED;
 }
 
@@ -266,7 +292,7 @@ test_rollback_inner (void *data)
   
   primitive_join_transaction (participant1);
   primitive_join_transaction (participant2);
-  primitive_abort_transaction (participant1);
+  primitive_abort_transaction (participant1, SCM_BOOL_F);
 
   gzochid_transaction_end ();
 
