@@ -1,5 +1,5 @@
 /* test-data.c: Test routines for data.c in gzochid.
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
  */
 
 #include <glib.h>
-#include <gmp.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
@@ -24,6 +23,8 @@
 #include "data.h"
 #include "game.h"
 #include "io.h"
+#include "oids.h"
+#include "oids-storage.h"
 #include "storage-mem.h"
 #include "tx.h"
 
@@ -33,24 +34,24 @@ static gboolean finalized = FALSE;
 
 static void 
 test_serializer 
-(gzochid_application_context *context, void *ptr, GString *out, GError **err)
+(gzochid_application_context *context, void *ptr, GByteArray *out, GError **err)
 {
   serialized = TRUE;
 }
 
 static void 
 test_failure_serializer
-(gzochid_application_context *context, void *ptr, GString *out, GError **err)
+(gzochid_application_context *context, void *ptr, GByteArray *out, GError **err)
 {
   g_set_error (err, GZOCHID_IO_ERROR, GZOCHID_IO_ERROR_SERIALIZATION,
 	       "Failed to serialize record.");
 }
 
 static void *test_deserializer 
-(gzochid_application_context *context, GString *in, GError **err)
+(gzochid_application_context *context, GByteArray *in, GError **err)
 {
   deserialized = TRUE;
-  return g_string_new_len (in->str, in->len);
+  return g_string_new_len (in->data, in->len);
 }
 
 static void test_finalizer (gzochid_application_context *context, void *ptr)
@@ -76,23 +77,18 @@ reset_serialization_state ()
 static void 
 fetch_reference (gpointer data)
 {
-  mpz_t z;
   gzochid_application_context *context = data;
   gzochid_data_managed_reference *ref = NULL;
 
-  mpz_init (z);
-
-  ref = gzochid_data_create_reference_to_oid (context, &test_serialization, z);
+  ref = gzochid_data_create_reference_to_oid (context, &test_serialization, 0);
   gzochid_data_dereference (ref, NULL);
-
-  mpz_clear (z);
 }
 
 static void 
 application_context_init (gzochid_application_context *context)
 {
   gzochid_context *base = (gzochid_context *) context;
-  gzochid_game_context *game_context = gzochid_game_context_new ();
+  gzochid_game_context *game_context = gzochid_game_context_new (NULL);
   base->parent = (gzochid_context *) game_context;
 
   game_context->storage_engine = malloc (sizeof (gzochid_storage_engine));
@@ -107,6 +103,10 @@ application_context_init (gzochid_application_context *context)
     (context->storage_context, "/dev/null", 0);
   context->names = gzochid_storage_engine_interface_mem.open 
     (context->storage_context, "/dev/null", 0);
+
+  context->oid_strategy = gzochid_storage_oid_strategy_new
+    (game_context->storage_engine->interface, context->storage_context,
+     context->meta);
 }
 
 static void
@@ -120,28 +120,30 @@ application_context_shutdown (gzochid_application_context *context)
   gzochid_storage_engine_interface_mem.close_store (context->oids);
   gzochid_storage_engine_interface_mem.close_store (context->names);
   gzochid_storage_engine_interface_mem.close_context (context->storage_context);
-					
+
+  gzochid_oid_allocation_strategy_free (context->oid_strategy);
+  
   gzochid_game_context_free (game_context);
 }
 
 static void test_data_reference_finalize ()
 {
-  mpz_t z;
-  char *key = NULL;
   gzochid_application_context *context = gzochid_application_context_new ();
-
+  gzochid_storage_transaction *tx = NULL;
+  guint64 zero = 0;
+  
   application_context_init (context);
   reset_serialization_state ();
 
-  mpz_init (z);
-  key = mpz_get_str (NULL, 16, z);
+  tx = gzochid_storage_engine_interface_mem.transaction_begin
+    (context->storage_context);
 
-  gzochid_storage_engine_interface_mem.put 
-    (context->oids, key, strlen (key) + 1, "foo", 4);
-
-  free (key);
-  mpz_clear (z);
+  gzochid_storage_engine_interface_mem.transaction_put 
+    (tx, context->oids, (char *) &zero, sizeof (guint64), "foo", 4);
 
+  gzochid_storage_engine_interface_mem.transaction_prepare (tx);  
+  gzochid_storage_engine_interface_mem.transaction_commit (tx);  
+  
   gzochid_transaction_execute (fetch_reference, context);
 
   g_assert (!serialized);
@@ -159,7 +161,7 @@ flush_reference_failure (gpointer data)
   gzochid_application_context *context = data;
 
   gzochid_data_create_reference
-    (context, &test_serialization_failure_serializer, str);
+    (context, &test_serialization_failure_serializer, str, NULL);
 }
 
 static void
