@@ -1,5 +1,5 @@
 /* test-scheme-task.c: Test routines for scheme-task.c in gzochid.
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
  */
 
 #include <glib.h>
-#include <gmp.h>
 #include <libguile.h>
 #include <stddef.h>
 #include <stdlib.h>
@@ -25,12 +24,16 @@
 #include "app-task.h"
 #include "context.h"
 #include "game.h"
+#include "guile.h"
 #include "gzochid-auth.h"
+#include "oids.h"
+#include "oids-storage.h"
 #include "scheme.h"
 #include "scheme-task.h"
 #include "storage-mem.h"
 #include "session.h"
 #include "tx.h"
+#include "util.h"
 
 struct test_scheme_task_fixture
 {
@@ -39,12 +42,22 @@ struct test_scheme_task_fixture
   gzochid_storage_engine_interface *storage_interface;
 };
 
+static gboolean
+ignore_warnings (const gchar *log_domain, GLogLevelFlags log_level,
+		 const gchar *message, gpointer user_data)
+{
+  if (log_level & G_LOG_LEVEL_CRITICAL
+      || log_level & G_LOG_LEVEL_WARNING)
+    return FALSE;
+  else return log_level & G_LOG_FLAG_FATAL;
+}
+
 static void
 test_scheme_task_fixture_setup (struct test_scheme_task_fixture *fixture,
 				gconstpointer user_data)
 {
   gzochid_context *base_context = NULL;
-  gzochid_game_context *game_context = gzochid_game_context_new ();
+  gzochid_game_context *game_context = gzochid_game_context_new (NULL);
   gzochid_storage_engine *storage_engine = 
     malloc (sizeof (gzochid_storage_engine));
 
@@ -72,6 +85,10 @@ test_scheme_task_fixture_setup (struct test_scheme_task_fixture *fixture,
     (fixture->context->storage_context, "/tmp/oids", GZOCHID_STORAGE_CREATE);
   fixture->context->names = fixture->storage_interface->open
     (fixture->context->storage_context, "/tmp/names", GZOCHID_STORAGE_CREATE);
+
+  fixture->context->oid_strategy = gzochid_storage_oid_strategy_new
+    (fixture->storage_interface, fixture->context->storage_context,
+     fixture->context->meta);
 }
 
 static void
@@ -89,62 +106,72 @@ test_scheme_task_fixture_teardown (struct test_scheme_task_fixture *fixture,
   fixture->storage_interface->close_store (fixture->context->names);
   fixture->storage_interface->close_context (fixture->context->storage_context);
 
+  gzochid_oid_allocation_strategy_free (fixture->context->oid_strategy);
+  
   free (game_context->storage_engine);
   gzochid_game_context_free (game_context);
   gzochid_application_context_free (fixture->context);
   gzochid_auth_identity_unref (fixture->identity);
 }
 
-static char *
+static void
 persist_client_session (struct test_scheme_task_fixture *fixture,
 			gzochid_client_session *session)
 {
-  mpz_t oid;
-  char *oid_str = NULL;
-  GString *out = g_string_new (NULL);
+  GByteArray *out = g_byte_array_new ();
+
+  gzochid_storage_transaction *tx = fixture->storage_interface
+    ->transaction_begin (fixture->context->storage_context);
   
   gzochid_client_session_serialization.serializer 
     (fixture->context, session, out, NULL);
   
-  mpz_init (oid);
-  mpz_set_ui (oid, 1);
-  oid_str = mpz_get_str (NULL, 16, oid);
-  
-  fixture->storage_interface->put
-    (fixture->context->oids, oid_str, strlen (oid_str) + 1, out->str, out->len);
-  gzochid_data_set_binding_to_oid (fixture->context, "s.session.1", oid, NULL);
-  
-  mpz_clear (oid);
-  g_string_free (out, TRUE);
+  fixture->storage_interface->transaction_put
+    (tx, fixture->context->oids, "1", 2, out->data, out->len);
+  fixture->storage_interface->transaction_put
+    (tx, fixture->context->names, "s.session.1", 12, "1", 2);
 
-  return oid_str;
+  fixture->storage_interface->transaction_prepare (tx);
+  fixture->storage_interface->transaction_commit (tx);
+
+  g_byte_array_unref (out);
 }
 
 static void 
 test_disconnected_worker_no_handler_inner (gpointer data)
 {
   struct test_scheme_task_fixture *fixture = data;
-  gzochid_client_session *session =
-    gzochid_client_session_new (fixture->identity);
-  char *oid_str = persist_client_session (fixture, session);
-
+  
+  g_test_log_set_fatal_handler (ignore_warnings, NULL);
+  
   gzochid_scheme_application_disconnected_worker
-    (fixture->context, fixture->identity, oid_str);
-
-  g_assert_null
-    (fixture->storage_interface->get
-     (fixture->context->oids, oid_str, strlen (oid_str) + 1, NULL));
-
-  free (oid_str);
-  gzochid_client_session_free (session);
+    (fixture->context, fixture->identity, "1");
 }
 
 static void 
 test_disconnected_worker_no_handler (struct test_scheme_task_fixture *fixture,
 				     gconstpointer user_data)
 {
+  guint64 one = gzochid_util_encode_oid (1);
+  gzochid_storage_transaction *tx = NULL;
+  gzochid_client_session *session =
+    gzochid_client_session_new (fixture->identity);
+  
+  persist_client_session (fixture, session);
+
   gzochid_transaction_execute
     (test_disconnected_worker_no_handler_inner, fixture);
+
+  tx = fixture->storage_interface->transaction_begin
+    (fixture->context->storage_context);
+
+  g_assert_null
+    (fixture->storage_interface->transaction_get
+     (tx, fixture->context->oids, (char *) &one, sizeof (guint64), NULL));
+
+  fixture->storage_interface->transaction_rollback (tx);
+
+  gzochid_client_session_free (session);
 }
 
 static SCM 
@@ -163,25 +190,13 @@ return_unspecified (SCM session)
 static gzochid_application_callback *
 make_callback (char *name, GList *module)
 {
-  mpz_t scm_oid;
-  gzochid_application_callback *callback = NULL;
-  
-  mpz_init (scm_oid);
-  mpz_set_si (scm_oid, -1);
-
-  callback = gzochid_application_callback_new (name, module, scm_oid);
-  mpz_clear (scm_oid);
-
-  return callback;
+  return gzochid_application_callback_new (name, module, -1);
 }
 
 static void 
 test_logged_in_worker_throws_exception_inner (gpointer data)
 {
   struct test_scheme_task_fixture *fixture = data;
-  gzochid_client_session *session =
-    gzochid_client_session_new (fixture->identity);
-  char *oid_str = persist_client_session (fixture, session);
   
   SCM module = scm_c_resolve_module ("test");
   SCM logged_in = scm_c_make_gsubr 
@@ -194,29 +209,31 @@ test_logged_in_worker_throws_exception_inner (gpointer data)
     ("logged-in-exception", test_module);
   
   gzochid_scheme_application_logged_in_worker
-    (fixture->context, fixture->identity, oid_str);
+    (fixture->context, fixture->identity, "1");
 
-  free (oid_str);
   g_list_free (test_module);
   gzochid_application_callback_free (fixture->context->descriptor->logged_in);
-  gzochid_client_session_free (session);
 }
 
 static void 
 test_logged_in_worker_throws_exception
 (struct test_scheme_task_fixture *fixture, gconstpointer user_data)
 {
+  gzochid_client_session *session =
+    gzochid_client_session_new (fixture->identity);
+
+  persist_client_session (fixture, session);
+
   gzochid_transaction_execute 
     (test_logged_in_worker_throws_exception_inner, fixture);
+
+  gzochid_client_session_free (session);
 }
 
 static void 
 test_logged_in_worker_returns_unspecified_inner (gpointer data)
 {
   struct test_scheme_task_fixture *fixture = data;
-  gzochid_client_session *session =
-    gzochid_client_session_new (fixture->identity);
-  char *oid_str = persist_client_session (fixture, session);
     
   SCM module = scm_c_resolve_module ("test");
   SCM logged_in = scm_c_make_gsubr 
@@ -229,20 +246,24 @@ test_logged_in_worker_returns_unspecified_inner (gpointer data)
     ("logged-in-unspecified", test_module);
 
   gzochid_scheme_application_logged_in_worker
-    (fixture->context, fixture->identity, oid_str);
+    (fixture->context, fixture->identity, "1");
 
-  free (oid_str);
   g_list_free (test_module);
   gzochid_application_callback_free (fixture->context->descriptor->logged_in);
-  gzochid_client_session_free (session);
 }
 
 static void 
 test_logged_in_worker_returns_unspecified
 (struct test_scheme_task_fixture *fixture, gconstpointer data)
 {
+  gzochid_client_session *session =
+    gzochid_client_session_new (fixture->identity);
+  persist_client_session (fixture, session);
+
   gzochid_transaction_execute 
     (test_logged_in_worker_returns_unspecified_inner, fixture);
+
+  gzochid_client_session_free (session);
 }
 
 static void 
