@@ -1,5 +1,5 @@
 /* test-scheme.c: Test routines for scheme.c in gzochid.
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -21,13 +21,34 @@
 #include <stdlib.h>
 
 #include "app.h"
+#include "channel.h"
+#include "guile.h"
+#include "gzochid-auth.h"
 #include "scheme.h"
+#include "session.h"
+#include "util.h"
+
+/* Fake implementation to avoid having to pull in `channel.o'. */
+
+const char *
+gzochid_channel_name (gzochid_channel *channel)
+{
+  return NULL;
+}
+
+/* Fake implementation to avoid having to pull in `session.o'. */
+
+gzochid_auth_identity *
+gzochid_client_session_identity (gzochid_client_session *session)
+{
+  return NULL;
+}
 
 static void
 test_serialization_exception ()
 {
   gzochid_application_context *context = gzochid_application_context_new ();
-  GString *str = g_string_new ("");
+  GByteArray *str = g_byte_array_new ();
   GError *err = NULL;
 
   context->descriptor = calloc (1, sizeof (gzochid_application_descriptor));
@@ -44,16 +65,16 @@ test_serialization_exception ()
 
   free (context->descriptor);
   gzochid_application_context_free (context);
-  g_string_free (str, TRUE);
+  g_byte_array_unref (str);
 }
 
 static void
 test_serialization_corrupt ()
 {
   gzochid_application_context *context = gzochid_application_context_new ();
-  GString *str = g_string_new ("");
+  GByteArray *str = g_byte_array_new ();
   GError *err = NULL;
-  
+
   gzochid_util_serialize_int (16, str);  
 
   g_assert
@@ -65,7 +86,7 @@ test_serialization_corrupt ()
 
   free (context->descriptor);
   gzochid_application_context_free (context);
-  g_string_free (str, TRUE);
+  g_byte_array_unref (str);
 }
 
 static void
