@@ -1,5 +1,5 @@
 /* test-session.c: Test routines for session.c in gzochid.
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -24,9 +24,11 @@
 #include "game.h"
 #include "guile.h"
 #include "scheme.h"
+#include "scheme-task.h"
 #include "session.h"
 #include "storage-mem.h"
 #include "tx.h"
+#include "util.h"
 
 static int
 prepare (gpointer data)
@@ -70,7 +72,7 @@ static void
 application_context_init (gzochid_application_context *context)
 {
   gzochid_context *base = (gzochid_context *) context;
-  gzochid_game_context *game_context = gzochid_game_context_new ();
+  gzochid_game_context *game_context = gzochid_game_context_new (NULL);
   base->parent = (gzochid_context *) game_context;
   
   game_context->storage_engine = malloc (sizeof (gzochid_storage_engine));
@@ -110,23 +112,29 @@ application_context_clear (gzochid_application_context *context)
 
 static void
 persist_callback (gzochid_application_context *context,
-		  gzochid_application_callback *callback, char *key,
-		  size_t key_len)
+		  gzochid_application_callback *callback, guint64 key)
 {
-  GString *callback_str = g_string_new ("");
+  GByteArray *callback_str = g_byte_array_new ();
   SCM scm_callback = gzochid_scheme_create_callback
     (callback, SCM_BOOL_F, NULL);
   GError *err = NULL;
+  gzochid_storage_transaction *tx = NULL;
+  guint64 encoded_oid = gzochid_util_encode_oid (key);
   
   gzochid_scheme_data_serialization.serializer
     (context, scm_callback, callback_str, &err);
 
-  gzochid_storage_engine_interface_mem.put
-    (context->oids, key, key_len, callback_str->str, callback_str->len);
+  tx = gzochid_storage_engine_interface_mem.transaction_begin
+    (context->storage_context);
+  gzochid_storage_engine_interface_mem.transaction_put
+    (tx, context->oids, (char *) &encoded_oid, sizeof (guint64),
+     (char *) callback_str->data, callback_str->len);
+  gzochid_storage_engine_interface_mem.transaction_prepare (tx);
+  gzochid_storage_engine_interface_mem.transaction_commit (tx);
 
-  mpz_set_str (callback->scm_oid, key, 16);
+  callback->scm_oid = key;
   
-  g_string_free (callback_str, TRUE);
+  g_byte_array_unref (callback_str);
   
 }
 
@@ -152,17 +160,18 @@ test_sweep_client_session_rollback_nonretryable ()
   GList *test_module = g_list_append (NULL, "test");
   gzochid_application_callback *disconnected_callback = NULL;
   gzochid_application_callback *received_message_callback = NULL;
-  mpz_t received_message_callback_arg_oid, disconnected_callback_arg_oid;
+
+  guint64 received_message_callback_arg_oid, disconnected_callback_arg_oid;
+  guint64 two = gzochid_util_encode_oid (2);
+  
+  gzochid_storage_transaction *tx = NULL;
   
   gzochid_client_session_handler handler; 
   
-  GString *serialized_session = g_string_new ("");
+  GByteArray *serialized_session = g_byte_array_new ();
 
   /* Context and test setup. */
   
-  mpz_init (received_message_callback_arg_oid);
-  mpz_init (disconnected_callback_arg_oid);
-  
   application_context_init (context);
 
   received_message_callback = gzochid_application_callback_new
@@ -170,20 +179,27 @@ test_sweep_client_session_rollback_nonretryable ()
   disconnected_callback = gzochid_application_callback_new
     ("disconnected", test_module, disconnected_callback_arg_oid);
 
-  persist_callback (context, received_message_callback, "0", 2);
-  persist_callback (context, disconnected_callback, "1", 2);
+  persist_callback (context, received_message_callback, 0);
+  persist_callback (context, disconnected_callback, 1);
   
   handler = (gzochid_client_session_handler)
     { received_message_callback, disconnected_callback };
-  session->handler = &handler;
+  gzochid_client_session_set_handler (session, &handler);
 
   gzochid_client_session_serialization.serializer
     (context, session, serialized_session, NULL);
 
-  gzochid_storage_engine_interface_mem.put
-    (context->oids, "2", 2, serialized_session->str, serialized_session->len);
-  gzochid_storage_engine_interface_mem.put
-    (context->names, "s.session.0", 12, "2", 2);
+  tx = gzochid_storage_engine_interface_mem.transaction_begin
+    (context->storage_context);
+
+  gzochid_storage_engine_interface_mem.transaction_put
+    (tx, context->oids, (char *) &two, sizeof (guint64),
+     (char *) serialized_session->data, serialized_session->len);
+  gzochid_storage_engine_interface_mem.transaction_put
+    (tx, context->names, "s.session.0", 12, (char *) &two, sizeof (guint64));
+
+  gzochid_storage_engine_interface_mem.transaction_prepare (tx);
+  gzochid_storage_engine_interface_mem.transaction_commit (tx);
 
   /* Execute the body of the test in a transaction. */
   
@@ -192,11 +208,8 @@ test_sweep_client_session_rollback_nonretryable ()
 
   /* Tear everything down. */
 
-  g_string_free (serialized_session, TRUE);
+  g_byte_array_unref (serialized_session);
   
-  mpz_clear (received_message_callback_arg_oid);
-  mpz_clear (disconnected_callback_arg_oid);
-
   gzochid_application_callback_free (received_message_callback);
   gzochid_application_callback_free (disconnected_callback);
   
