@@ -1,5 +1,5 @@
 /* test-storage-mem.c: Test routines for storage-mem.c in gzochid.
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 #include <assert.h>
 #include <glib.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "gzochid-storage.h"
 #include "storage-mem.h"
@@ -28,6 +29,16 @@ struct test_storage_fixture
   gzochid_storage_store *store;
 };
 
+static gboolean
+ignore_warnings (const gchar *log_domain, GLogLevelFlags log_level,
+                 const gchar *message, gpointer user_data)
+{
+  if (log_level & G_LOG_LEVEL_CRITICAL
+      || log_level & G_LOG_LEVEL_WARNING)
+    return FALSE;
+  else return log_level & G_LOG_FLAG_FATAL;
+}
+
 static void
 test_storage_fixture_setup
 (struct test_storage_fixture *fixture, gconstpointer user_data)
@@ -70,112 +81,6 @@ test_storage_mem_open ()
   gzochid_storage_engine_interface_mem.close_context (context);
 }
 
-static void
-test_storage_mem_get
-(struct test_storage_fixture *fixture, gconstpointer user_data)
-{
-  size_t value_len = 0;
-  char *value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, "foo", 4, &value_len);
-
-  g_assert_null (value);
-  g_assert_cmpint (value_len, ==, 0);
-  
-  gzochid_storage_engine_interface_mem.put (fixture->store, "foo", 4, "bar", 4);
-
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, "foo", 4, &value_len);
-  
-  g_assert_cmpstr (value, ==, "bar");
-  g_assert_cmpint (value_len, ==, 4);
-
-  free (value);
-}
-
-static void
-test_storage_mem_put
-(struct test_storage_fixture *fixture, gconstpointer user_data)
-{
-  char *value = NULL;
-  size_t value_len = 0;
-
-  gzochid_storage_engine_interface_mem.put (fixture->store, "foo", 4, "bar", 4);
-  gzochid_storage_engine_interface_mem.put (fixture->store, "foo", 4, "baz", 4);
-
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, "foo", 4, &value_len);
-  
-  g_assert_cmpstr (value, ==, "baz");
-  g_assert_cmpint (value_len, ==, 4);
-
-  free (value);
-}
-
-static void
-test_storage_mem_delete
-(struct test_storage_fixture *fixture, gconstpointer user_data)
-{
-  char *value = NULL;
-  size_t value_len = 0;
-
-  gzochid_storage_engine_interface_mem.put (fixture->store, "foo", 4, "bar", 4);
-  gzochid_storage_engine_interface_mem.delete (fixture->store, "foo", 4);
-
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, "foo", 4, &value_len);
-  
-  g_assert_null (value);
-}
-
-static void
-test_storage_mem_first_key
-(struct test_storage_fixture *fixture, gconstpointer user_data)
-{
-  size_t key_len = 0;
-  char *key = gzochid_storage_engine_interface_mem.first_key 
-    (fixture->store, &key_len);
-
-  g_assert_null (key);
-  g_assert_cmpint (key_len, ==, 0);
-  
-  gzochid_storage_engine_interface_mem.put (fixture->store, "foo", 4, "bar", 4);
-  gzochid_storage_engine_interface_mem.put 
-    (fixture->store, "baz", 4, "quux", 5);
-
-  key = gzochid_storage_engine_interface_mem.first_key 
-    (fixture->store, &key_len);
-  
-  g_assert_cmpstr (key, ==, "baz");
-  g_assert_cmpint (key_len, ==, 4);
-
-  free (key);
-}
-
-static void
-test_storage_mem_next_key
-(struct test_storage_fixture *fixture, gconstpointer user_data)
-{
-  char *key = NULL;
-  size_t key_len = 0;
-
-  gzochid_storage_engine_interface_mem.put (fixture->store, "foo", 4, "bar", 4);
-  gzochid_storage_engine_interface_mem.put 
-    (fixture->store, "baz", 4, "quux", 5);
-
-  key = gzochid_storage_engine_interface_mem.next_key 
-    (fixture->store, "baz", 4, &key_len);
-  
-  g_assert_cmpstr (key, ==, "foo");
-  g_assert_cmpint (key_len, ==, 4);
-
-  free (key);
-
-  key = gzochid_storage_engine_interface_mem.next_key 
-    (fixture->store, "foo", 4, &key_len);
-
-  g_assert_null (key);
-}
-
 static void
 test_storage_mem_tx_put_get_commit_get
 (struct test_storage_fixture *fixture, gconstpointer user_data)
@@ -197,10 +102,17 @@ test_storage_mem_tx_put_get_commit_get
   gzochid_storage_engine_interface_mem.transaction_prepare (tx);
   gzochid_storage_engine_interface_mem.transaction_commit (tx);
 
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, "foo", 4, &value_len);
+  tx = gzochid_storage_engine_interface_mem.transaction_begin
+    (fixture->context);
+  
+  value = gzochid_storage_engine_interface_mem.transaction_get 
+    (tx, fixture->store, "foo", 4, &value_len);
+
+  gzochid_storage_engine_interface_mem.transaction_rollback (tx);
+
   g_assert_cmpstr (value, ==, "bar");
   g_assert_cmpint (value_len, ==, 4);  
+
   free (value);
 }
 
@@ -223,8 +135,14 @@ test_storage_mem_tx_put_rollback_get
   gzochid_storage_engine_interface_mem.transaction_rollback (tx);
   free (value);
 
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, "foo", 4, &value_len);
+  tx = gzochid_storage_engine_interface_mem.transaction_begin
+    (fixture->context);
+
+  value = gzochid_storage_engine_interface_mem.transaction_get 
+    (tx, fixture->store, "foo", 4, &value_len);
+
+  gzochid_storage_engine_interface_mem.transaction_rollback (tx);
+  
   g_assert_null (value);
 }
 
@@ -251,9 +169,14 @@ test_storage_mem_tx_delete_get_commit_get
 
   gzochid_storage_engine_interface_mem.transaction_commit (tx);
 
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, "foo", 4, &value_len);
+  tx = gzochid_storage_engine_interface_mem.transaction_begin
+    (fixture->context);
+
+  value = gzochid_storage_engine_interface_mem.transaction_get 
+    (tx, fixture->store, "foo", 4, &value_len);
   g_assert_null (value);
+
+  gzochid_storage_engine_interface_mem.transaction_rollback (tx);
 }
 
 static void
@@ -262,9 +185,14 @@ test_storage_mem_tx_delete_rollback_get
 {
   char *value = NULL;
   size_t value_len = 0;
-  gzochid_storage_transaction *tx = NULL;
+  gzochid_storage_transaction *tx =
+    gzochid_storage_engine_interface_mem.transaction_begin (fixture->context);
+    
+  gzochid_storage_engine_interface_mem.transaction_put
+    (tx, fixture->store, "foo", 4, "bar", 4);
 
-  gzochid_storage_engine_interface_mem.put (fixture->store, "foo", 4, "bar", 4);
+  gzochid_storage_engine_interface_mem.transaction_prepare (tx);
+  gzochid_storage_engine_interface_mem.transaction_commit (tx);
 
   tx = gzochid_storage_engine_interface_mem.transaction_begin 
     (fixture->context);
@@ -272,11 +200,18 @@ test_storage_mem_tx_delete_rollback_get
     (tx, fixture->store, "foo", 4);
   gzochid_storage_engine_interface_mem.transaction_rollback (tx);
 
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, "foo", 4, &value_len);
+  tx = gzochid_storage_engine_interface_mem.transaction_begin
+    (fixture->context);
+
+  value = gzochid_storage_engine_interface_mem.transaction_get
+    (tx, fixture->store, "foo", 4, &value_len);
+
   g_assert_cmpstr (value, ==, "bar");
   g_assert_cmpint (value_len, ==, 4);
   free (value);
+
+  gzochid_storage_engine_interface_mem.transaction_rollback (tx);
+
 }
 
 struct test_storage_fixture_concurrent
@@ -347,7 +282,7 @@ deadlock_simple_thread_1 (gpointer data)
     gzochid_storage_engine_interface_mem.transaction_begin 
     (fixture->base_fixture->context);
   char *value1 = gzochid_storage_engine_interface_mem.transaction_get 
-    (tx1, fixture->base_fixture->store, "foo", 4, NULL);
+    (tx1, fixture->base_fixture->store, "\001", 1, NULL);
 
   free (value1);
 
@@ -355,7 +290,7 @@ deadlock_simple_thread_1 (gpointer data)
   wait_latch_zero (fixture);
   
   gzochid_storage_engine_interface_mem.transaction_put 
-    (tx1, fixture->base_fixture->store, "baz", 4, "quux2", 6);
+    (tx1, fixture->base_fixture->store, "\047", 1, "quux2", 6);
 
   if (tx1->rollback)
     fixture->tx1_rollback = TRUE;
@@ -373,7 +308,7 @@ deadlock_simple_thread_2 (gpointer data)
     gzochid_storage_engine_interface_mem.transaction_begin 
     (fixture->base_fixture->context);
   char *value2 = gzochid_storage_engine_interface_mem.transaction_get 
-    (tx2, fixture->base_fixture->store, "baz", 4, NULL);
+    (tx2, fixture->base_fixture->store, "\047", 1, NULL);
 
   free (value2);
 
@@ -381,7 +316,7 @@ deadlock_simple_thread_2 (gpointer data)
   wait_latch_zero (fixture);
 
   gzochid_storage_engine_interface_mem.transaction_put 
-    (tx2, fixture->base_fixture->store, "foo", 4, "bar2", 5);
+    (tx2, fixture->base_fixture->store, "\001", 1, "bar2", 5);
 
   if (tx2->rollback)
     fixture->tx2_rollback = TRUE;
@@ -398,11 +333,29 @@ test_storage_mem_tx_deadlock_simple
   GThread *thread1 = NULL;
   GThread *thread2 = NULL;
 
-  gzochid_storage_engine_interface_mem.put 
-    (fixture->base_fixture->store, "foo", 4, "bar", 4);
-  gzochid_storage_engine_interface_mem.put 
-    (fixture->base_fixture->store, "baz", 4, "quux", 5);
+  gzochid_storage_transaction *tx =
+    gzochid_storage_engine_interface_mem.transaction_begin
+    (fixture->base_fixture->context);
 
+  int i = 0;
+  char c = 0;
+  char buf[1024];
+  
+  char *value = NULL;
+  size_t value_len = 0;
+
+  for (; i < 40; i++)
+    {
+      c = (char) i;
+      memset (buf, c, 1024);
+      gzochid_storage_engine_interface_mem.transaction_put 
+	(tx, fixture->base_fixture->store, &c, 1, buf, 1024);
+    }
+  
+  gzochid_storage_engine_interface_mem.transaction_commit (tx);
+
+  g_test_log_set_fatal_handler (ignore_warnings, NULL);
+  
   fixture->latch = 2;
 
   thread1 = g_thread_new
@@ -422,38 +375,46 @@ test_storage_mem_tx_split_root
 {
   int i = 0;
   char c = 0;
-
+  char buf[1024];
+  
   char *value = NULL;
   size_t value_len = 0;
   gzochid_storage_transaction *tx = 
     gzochid_storage_engine_interface_mem.transaction_begin (fixture->context);
 
-  for (; i < 10; i++)
+  for (; i < 40; i++)
     {
       c = (char) i;
+      memset (buf, c, 1024);
       gzochid_storage_engine_interface_mem.transaction_put 
-	(tx, fixture->store, &c, 1, &c, 1);
+	(tx, fixture->store, &c, 1, buf, 1024);
     }
   
   gzochid_storage_engine_interface_mem.transaction_commit (tx);
-
+  
+  tx = gzochid_storage_engine_interface_mem.transaction_begin
+    (fixture->context);
+  
   c = 0;
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, &c, 1, &value_len);
+  value = gzochid_storage_engine_interface_mem.transaction_get 
+    (tx, fixture->store, &c, 1, &value_len);
 
   g_assert_nonnull (value);
   g_assert_cmpint (*value, ==, 0);
-  g_assert_cmpint (value_len, ==, 1);
+  g_assert_cmpint (value_len, ==, 1024);
   free (value);
 
-  c = 9;
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, &c, 1, &value_len);
+  c = 39;
+  value = gzochid_storage_engine_interface_mem.transaction_get 
+    (tx, fixture->store, &c, 1, &value_len);
 
   g_assert_nonnull (value);
-  g_assert_cmpint (*value, ==, 9);
-  g_assert_cmpint (value_len, ==, 1);
+  g_assert_cmpint (*value, ==, 39);
+  g_assert_cmpint (value_len, ==, 1024);
   free (value);
+
+  gzochid_storage_engine_interface_mem.transaction_rollback (tx);
+
 }
 
 static void
@@ -462,44 +423,54 @@ test_storage_mem_tx_split_internal
 {
   int i = 0;
   char c = 0;
+  char buf[1024];
 
   char *value = NULL;
   size_t value_len = 0;
   gzochid_storage_transaction *tx = 
     gzochid_storage_engine_interface_mem.transaction_begin (fixture->context);
 
-  for (; i < 10; i++)
+  for (; i < 40; i++)
     {
       c = (char) i * 10;
+      memset (buf, c, 1024);
+      
       gzochid_storage_engine_interface_mem.transaction_put 
-	(tx, fixture->store, &c, 1, &c, 1);
+	(tx, fixture->store, &c, 1, buf, 1024);
     }
-  for (i = 1; i < 10; i++)
+  for (i = 1; i < 40; i++)
     {
       c = (char) i;
+      memset (buf, c, 1024);
+
       gzochid_storage_engine_interface_mem.transaction_put 
-	(tx, fixture->store, &c, 1, &c, 1);
+	(tx, fixture->store, &c, 1, buf, 1024);
     }
   
   gzochid_storage_engine_interface_mem.transaction_commit (tx);
 
+  tx = gzochid_storage_engine_interface_mem.transaction_begin
+    (fixture->context);
+
   c = 1;
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, &c, 1, &value_len);
+  value = gzochid_storage_engine_interface_mem.transaction_get 
+    (tx, fixture->store, &c, 1, &value_len);
 
   g_assert_nonnull (value);
   g_assert_cmpint (*value, ==, 1);
-  g_assert_cmpint (value_len, ==, 1);
+  g_assert_cmpint (value_len, ==, 1024);
   free (value);
 
   c = 90;
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, &c, 1, &value_len);
+  value = gzochid_storage_engine_interface_mem.transaction_get 
+    (tx, fixture->store, &c, 1, &value_len);
 
   g_assert_nonnull (value);
   g_assert_cmpint (*value, ==, 90);
-  g_assert_cmpint (value_len, ==, 1);
+  g_assert_cmpint (value_len, ==, 1024);
   free (value);
+
+  gzochid_storage_engine_interface_mem.transaction_rollback (tx);
 }
 
 static void
@@ -532,9 +503,12 @@ test_storage_mem_tx_merge_internal
   
   gzochid_storage_engine_interface_mem.transaction_commit (tx);
 
+  tx = gzochid_storage_engine_interface_mem.transaction_begin
+    (fixture->context);
+
   c = 0;
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, &c, 1, &value_len);
+  value = gzochid_storage_engine_interface_mem.transaction_get 
+    (tx, fixture->store, &c, 1, &value_len);
 
   g_assert_nonnull (value);
   g_assert_cmpint (*value, ==, 0);
@@ -542,13 +516,15 @@ test_storage_mem_tx_merge_internal
   free (value);
 
   c = 19;
-  value = gzochid_storage_engine_interface_mem.get 
-    (fixture->store, &c, 1, &value_len);
+  value = gzochid_storage_engine_interface_mem.transaction_get 
+    (tx, fixture->store, &c, 1, &value_len);
 
   g_assert_nonnull (value);
   g_assert_cmpint (*value, ==, 19);
   g_assert_cmpint (value_len, ==, 1);
   free (value);
+
+  gzochid_storage_engine_interface_mem.transaction_rollback (tx);  
 }
 
 int
@@ -559,22 +535,6 @@ main (int argc, char *argv[])
   g_test_add_func ("/storage-mem/initialize", test_storage_mem_initialize);
   g_test_add_func ("/storage-mem/open", test_storage_mem_open);
 
-  g_test_add ("/storage-mem/get", struct test_storage_fixture, NULL, 
-	      test_storage_fixture_setup, test_storage_mem_get, 
-	      test_storage_fixture_teardown);
-  g_test_add ("/storage-mem/put", struct test_storage_fixture, NULL, 
-	      test_storage_fixture_setup, test_storage_mem_put, 
-	      test_storage_fixture_teardown);
-  g_test_add ("/storage-mem/delete", struct test_storage_fixture, NULL, 
-	      test_storage_fixture_setup, test_storage_mem_delete, 
-	      test_storage_fixture_teardown);
-  g_test_add ("/storage-mem/first-key", struct test_storage_fixture, NULL, 
-	      test_storage_fixture_setup, test_storage_mem_first_key, 
-	      test_storage_fixture_teardown);
-  g_test_add ("/storage-mem/next-key", struct test_storage_fixture, NULL, 
-	      test_storage_fixture_setup, test_storage_mem_next_key, 
-	      test_storage_fixture_teardown);
-
   g_test_add
     ("/storage-mem/tx/put-get-commit-get", struct test_storage_fixture, NULL, 
      test_storage_fixture_setup, test_storage_mem_tx_put_get_commit_get, 
